@@ -2,12 +2,12 @@
 
 MAKE=make
 
-UIM_REPOSITORY="http://anonsvn.freedesktop.org/svn/uim"
+UIM_REPOSITORY="http://uim.googlecode.com/svn"
 TAGS_REPOSITORY="${UIM_REPOSITORY}/tags"
 #SSCM_REPOSITORY="${UIM_REPOSITORY}/sigscheme-trunk"
-SSCM_REPOSITORY="${TAGS_REPOSITORY}/sigscheme-0.7.4"
+SSCM_REPOSITORY="${TAGS_REPOSITORY}/sigscheme-0.8.0"
 #LIBGCROOTS_URL="${UIM_REPOSITORY}/libgcroots-trunk"
-LIBGCROOTS_URL="${TAGS_REPOSITORY}/libgcroots-0.1.4"
+LIBGCROOTS_URL="${TAGS_REPOSITORY}/libgcroots-0.2.0"
 
 svn export $LIBGCROOTS_URL libgcroots
 (cd libgcroots && ./autogen.sh) \
