@@ -2,7 +2,10 @@
  *  Filename : encoding.c
  *  About    : Character encoding handling
  *
- *  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
+ *  Copyright (C) 2005      Kazuki Ohta <mover AT hct.zaq.ne.jp>
+ *  Copyright (C) 2005      Jun Inoue <jun.lambda AT gmail.com>
+ *  Copyright (C) 2005-2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -44,7 +47,6 @@
 #include <config.h>
 
 #include <stdlib.h>
-#include <stdio.h>  /* for EOF */
 #include <string.h>
 
 #include "scmint.h"
@@ -360,7 +362,7 @@ scm_charcodec_read_char(ScmCharCodec *codec, ScmMultibyteString *mbs,
                                SCM_MBS_GET_STR(*mbs),
                                SCM_MBCINFO_GET_SIZE(mbc),
                                state);
-    if (ch == EOF)
+    if (ch == SCM_ICHAR_EOF)
         SCM_ENCODING_ERROR("scm_charcodec_read_char: invalid char sequence");
 
     SCM_MBS_SKIP_CHAR(*mbs, mbc);
@@ -458,7 +460,7 @@ static int
 eucjp_char_len(scm_ichar_t ch)
 {
     uchar *end;
-    uchar buf[SCM_MB_MAX_LEN + sizeof("")];
+    uchar buf[SCM_MB_CHAR_BUF_SIZE];
 
     end = eucjp_int2str(buf, ch, SCM_MB_STATELESS);
 
@@ -535,7 +537,7 @@ eucjp_str2int(const uchar *src, size_t len, ScmMultibyteState state)
         break;
 
     default:
-        return EOF;
+        return SCM_ICHAR_EOF;
     }
 
     return ch;
@@ -601,7 +603,7 @@ dbc_str2int(const uchar *src, size_t len, ScmMultibyteState state)
         break;
 
     default:
-        return EOF;
+        return SCM_ICHAR_EOF;
     }
 
     return ch;
@@ -614,7 +616,7 @@ static int
 euc_char_len(scm_ichar_t ch)
 {
     uchar *end;
-    uchar buf[SCM_MB_MAX_LEN + sizeof("")];
+    uchar buf[SCM_MB_CHAR_BUF_SIZE];
 
     end = euc_int2str(buf, ch, SCM_MB_STATELESS);
 
@@ -749,7 +751,7 @@ euckr_scan_char(ScmMultibyteString mbs)
 #if SCM_USE_UTF8
 /* RFC 3629 */
 #define MASK(n)        ((LEN_CODE(n) >> 1) | 0x80)
-#define LEN_CODE(n)    (((1 << (n))-1) << (8-n))
+#define LEN_CODE(n)    (((1 << (n)) - 1) << (8 - n))
 #define IS_LEN(c, n)   ((MASK(n) & (c)) == LEN_CODE(n))
 #define IS_TRAILING(c) (IS_LEN((c), 1))
 
@@ -769,7 +771,7 @@ utf8_encoding(void)
 static enum ScmCodedCharSet
 utf8_ccs(void)
 {
-    return SCM_CCS_UCS4;
+    return SCM_CCS_UNICODE;
 }
 
 /* FIXME: Optimize */
@@ -777,7 +779,7 @@ static int
 utf8_char_len(scm_ichar_t ch)
 {
     uchar *end;
-    uchar buf[SCM_MB_MAX_LEN + sizeof("")];
+    uchar buf[SCM_MB_CHAR_BUF_SIZE];
 
     end = utf8_int2str(buf, ch, SCM_MB_STATELESS);
 
@@ -850,7 +852,7 @@ utf8_str2int(const uchar *src, size_t len, ScmMultibyteState state)
         break;
 
     default:
-        return EOF;
+        return SCM_ICHAR_EOF;
     }
 
     return ch;
@@ -921,13 +923,13 @@ utf8_int2str(uchar *dst, scm_ichar_t ch, ScmMultibyteState state)
 static const char *
 sjis_encoding(void)
 {
-    return "SHIFT_JIS";
+    return "Shift_JIS";
 }
 
 static enum ScmCodedCharSet
 sjis_ccs(void)
 {
-    return SCM_CCS_UNKNOWN;
+    return SCM_CCS_JIS;
 }
 
 /* FIXME: Optimize */
@@ -935,7 +937,7 @@ static int
 sjis_char_len(scm_ichar_t ch)
 {
     uchar *end;
-    uchar buf[SCM_MB_MAX_LEN + sizeof("")];
+    uchar buf[SCM_MB_CHAR_BUF_SIZE];
 
     end = sjis_int2str(buf, ch, SCM_MB_STATELESS);
 
@@ -1036,7 +1038,7 @@ unibyte_str2int(const uchar *src, size_t len, ScmMultibyteState state)
 {
 #if SCM_STRICT_ENCODING_CHECK
     if (len != 1)
-        return EOF;
+        return SCM_ICHAR_EOF;
 #endif
     return src[0];
 }
@@ -1049,5 +1051,7 @@ unibyte_int2str(uchar *dst, scm_ichar_t ch, ScmMultibyteState state)
         return NULL;
 #endif
     *dst++ = ch;
+    *dst = '\0';
+
     return dst;
 }
