@@ -2,7 +2,10 @@
  *  Filename : eval.c
  *  About    : Evaluation and function calling
  *
- *  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
+ *  Copyright (C) 2005      Kazuki Ohta <mover AT hct.zaq.ne.jp>
+ *  Copyright (C) 2005-2006 Jun Inoue <jun.lambda AT gmail.com>
+ *  Copyright (C) 2005-2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -40,8 +43,7 @@
 /*=======================================
   File Local Macro Definitions
 =======================================*/
-#define SCM_ERRMSG_WRONG_NR_ARG " Wrong number of arguments "
-#define SCM_ERRMSG_NON_R5RS_ENV " the environment is not conformed to R5RS"
+#define SCM_ERRMSG_NON_R5RS_ENV "the environment is not conformed to R5RS"
 
 /*=======================================
   File Local Type Definitions
@@ -56,9 +58,11 @@
 =======================================*/
 static ScmObj reduce(ScmObj (*func)(), ScmObj args, ScmObj env,
                      enum ScmValueType need_eval);
+#if SCM_USE_CONTINUATION
 static void call_continuation(ScmObj cont, ScmObj args,
                               ScmEvalState *eval_state,
                               enum ScmValueType need_eval) SCM_NORETURN;
+#endif
 static ScmObj call_closure(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
                            enum ScmValueType need_eval);
 static ScmObj call(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
@@ -68,16 +72,6 @@ static ScmObj map_eval(ScmObj args, scm_int_t *args_len, ScmObj env);
 /*=======================================
   Function Definitions
 =======================================*/
-/* A wrapper for call() for internal proper tail recursion */
-SCM_EXPORT ScmObj
-scm_tailcall(ScmObj proc, ScmObj args, ScmEvalState *eval_state)
-{
-    SCM_ASSERT(PROPER_LISTP(args));
-
-    eval_state->ret_type = SCM_VALTYPE_AS_IS;
-    return call(proc, args, eval_state, SCM_VALTYPE_AS_IS);
-}
-
 /* Wrapper for call().  Just like scm_p_apply(), except ARGS is used
  * as given---nothing special is done about the last item in the
  * list. */
@@ -104,8 +98,7 @@ scm_call(ScmObj proc, ScmObj args)
 static ScmObj
 reduce(ScmObj (*func)(), ScmObj args, ScmObj env, enum ScmValueType need_eval)
 {
-    ScmObj left;
-    ScmObj right;
+    ScmObj left, right;
     enum ScmReductionState state;
     DECLARE_INTERNAL_FUNCTION("(reduction)");
 
@@ -141,27 +134,34 @@ reduce(ScmObj (*func)(), ScmObj args, ScmObj env, enum ScmValueType need_eval)
     return (*func)(left, right, &state);
 }
 
+#if SCM_USE_CONTINUATION
 static void
 call_continuation(ScmObj cont, ScmObj args, ScmEvalState *eval_state,
                   enum ScmValueType need_eval)
 {
     ScmObj ret;
+    scm_int_t args_len;
     DECLARE_INTERNAL_FUNCTION("call_continuation");
 
-    if (!LIST_1_P(args))
-        ERR_OBJ("continuation takes exactly one argument but got", args);
-    ret = CAR(args);
-    if (need_eval)
-        ret = EVAL(ret, eval_state->env);
+    /* (receive (x y) (call/cc (lambda (k) (k 0 1)))) */
+    if (LIST_1_P(args)) {
+        ret = CAR(args);
+        if (need_eval)
+            ret = EVAL(ret, eval_state->env);
+    } else {
+        ret = (need_eval) ? map_eval(args, &args_len, eval_state->env) : args;
+        ret = SCM_MAKE_VALUEPACKET(ret);
+    }
     scm_call_continuation(cont, ret);
     /* NOTREACHED */
 }
+#endif /* SCM_USE_CONTINUATION */
 
 static ScmObj
 call_closure(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
              enum ScmValueType need_eval)
 {
-    ScmObj formals, body, proc_env;
+    ScmObj exp, formals, body, proc_env;
     scm_int_t formals_len, args_len;
     DECLARE_INTERNAL_FUNCTION("call_closure");
 
@@ -176,8 +176,9 @@ call_closure(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
      *   (2) (<variable1> <variable2> ...)
      *   (3) (<variable1> <variable2> ... <variable n-1> . <variable n>)
      */
-    formals  = CAR(SCM_CLOSURE_EXP(proc));
-    body     = CDR(SCM_CLOSURE_EXP(proc));
+    exp      = SCM_CLOSURE_EXP(proc);
+    formals  = CAR(exp);
+    body     = CDR(exp);
     proc_env = SCM_CLOSURE_ENV(proc);
     if (need_eval) {
         args = map_eval(args, &args_len, eval_state->env);
@@ -189,9 +190,8 @@ call_closure(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
 
     if (IDENTIFIERP(formals)) {
         /* (1) <variable> */
-        eval_state->env = scm_extend_environment(LIST_1(formals),
-                                                 LIST_1(args),
-                                                 proc_env);
+        formals = LIST_1(formals);
+        args    = LIST_1(args);
     } else if (CONSP(formals)) {
         /*
          * (2) (<variable1> <variable2> ...)
@@ -204,8 +204,6 @@ call_closure(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
         formals_len = scm_finite_length(formals);
         if (!scm_valid_environment_extension_lengthp(formals_len, args_len))
             goto err_improper;
-
-        eval_state->env = scm_extend_environment(formals, args, proc_env);
     } else if (NULLP(formals)) {
         /*
          * (2') <variable> is '()
@@ -213,11 +211,12 @@ call_closure(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
         if (args_len)
             goto err_improper;
 
-        eval_state->env = scm_extend_environment(SCM_NULL, SCM_NULL, proc_env);
+        formals = args = SCM_NULL;
     } else {
-        SCM_ASSERT(scm_false);
+        SCM_NOTREACHED;
     }
 
+    eval_state->env = scm_extend_environment(formals, args, proc_env);
     eval_state->ret_type = SCM_VALTYPE_NEED_EVAL;
     return scm_s_body(body, eval_state);
 
@@ -227,24 +226,21 @@ call_closure(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
 
 /**
  * @param proc The procedure or syntax to call.
- *
  * @param args The argument list.
- *
  * @param eval_state The calling evaluator's state.
- *
  * @param need_eval Indicates that @a args need be evaluated.
  */
 static ScmObj
 call(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
      enum ScmValueType need_eval)
 {
-    ScmObj env;
+    ScmObj env, vals;
     ScmObj (*func)();
     enum ScmFuncTypeCode type;
     scm_bool syntaxp;
     int mand_count, i;
     scm_int_t variadic_len;
-    /* The +2 is for rest and env/eval_state. */
+    /* The +2 is for rest and env. */
     ScmObj argbuf[SCM_FUNCTYPE_MAND_MAX + 2];
     DECLARE_INTERNAL_FUNCTION("(function call)");
 
@@ -253,9 +249,40 @@ call(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
     if (need_eval)
         proc = EVAL(proc, env);
 
-    if (!FUNCP(proc)) {
-        if (CLOSUREP(proc))
-            return call_closure(proc, args, eval_state, need_eval);
+    while (!FUNCP(proc)) {
+        if (CLOSUREP(proc)) {
+#if SCM_USE_LEGACY_MACRO
+            if (SYNTACTIC_CLOSUREP(proc)) {
+                ScmObj ret;
+                scm_bool toplevelp;
+
+                if (!need_eval)
+                    ERR_OBJ("can't apply/map a macro", proc);
+
+                toplevelp = SCM_DEFINABLE_TOPLEVELP(eval_state);
+
+                ret = call_closure(proc, args, eval_state, SCM_VALTYPE_AS_IS);
+                /* eval the result into an as-is object */
+                ret = SCM_FINISH_TAILREC_CALL(ret, eval_state);
+                /* restore previous env */
+                eval_state->env = env;
+                /* Instruct evaluating returned object again as a syntactic
+                 * form. */
+                eval_state->ret_type = SCM_VALTYPE_NEED_EVAL;
+#if SCM_STRICT_TOPLEVEL_DEFINITIONS
+                /* Workaround to allow toplevel definitions by the returned
+                 * form. See scm_eval(). */
+                if (toplevelp)
+                    eval_state->nest = SCM_NEST_RETTYPE_BEGIN;
+#endif
+
+                return ret;
+            } else
+#endif /* SCM_USE_LEGACY_MACRO */
+            {
+                return call_closure(proc, args, eval_state, need_eval);
+            }
+        }
 #if SCM_USE_HYGIENIC_MACRO
         if (HMACROP(proc)) {
             if (!need_eval)
@@ -263,11 +290,35 @@ call(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
             return scm_expand_macro(proc, args, eval_state);
         }
 #endif
+        /* Since scm_values_applier is a continuation, this block must precedes
+         * CONTINUATIONP(). */
+        if (EQ(proc, scm_values_applier)) {
+            if (!need_eval)
+                ERR("invalid multiple values application");
+            proc = MUST_POP_ARG(args);
+            vals = MUST_POP_ARG(args);
+            NO_MORE_ARG(args);
+
+            if (!VALUEPACKETP(vals)) {
+                /* got back a single value */
+                args = LIST_1(vals);
+            } else {
+                /* extract */
+                args = SCM_VALUEPACKET_VALUES(vals);
+            }
+            /* the values and the consumer must be both already evaluated
+             * though need_eval == scm_true */
+            need_eval = scm_false;
+            continue;
+        }
+#if SCM_USE_CONTINUATION
         if (CONTINUATIONP(proc)) {
             call_continuation(proc, args, eval_state, need_eval);
             /* NOTREACHED */
         }
+#endif
         ERR_OBJ("procedure or syntax required but got", proc);
+        /* NOTREACHED */
     }
 
     /* We have a C function. */
@@ -293,10 +344,7 @@ call(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
         argbuf[i] = MUST_POP_ARG(args);
         if (need_eval)
             argbuf[i] = EVAL(argbuf[i], env);
-#if SCM_STRICT_ARGCHECK
-        if (VALUEPACKETP((ScmObj)argbuf[i]))
-            ERR_OBJ("multiple values are not allowed here", (ScmObj)argbuf[i]);
-#endif
+        CHECK_VALID_EVALED_VALUE((ScmObj)argbuf[i]);
     }
 
     if (type & SCM_FUNCTYPE_VARIADIC) {
@@ -315,64 +363,99 @@ call(ScmObj proc, ScmObj args, ScmEvalState *eval_state,
 
     if (type & SCM_FUNCTYPE_TAILREC) {
         eval_state->ret_type = SCM_VALTYPE_NEED_EVAL;
+#if (SIZEOF_VOID_P != SIZEOF_SCMOBJ)
+        /* eval_state cannot be stored into argbuf safely. */
+        argbuf[i++] = SCM_INVALID; /* dummy */
+#else
         argbuf[i++] = (ScmObj)eval_state;
+#endif
     } else {
         eval_state->ret_type = SCM_VALTYPE_AS_IS;
         if (type & SCM_FUNCTYPE_SYNTAX)
             argbuf[i++] = env;
     }
 
+#if (SIZEOF_VOID_P != SIZEOF_SCMOBJ)
+    if (type & SCM_FUNCTYPE_TAILREC) {
+        switch (i) {
+        case 1:
+            return (*func)(eval_state);
+        case 2:
+            return (*func)(argbuf[0], eval_state);
+        case 3:
+#if SCM_FUNCTYPE_MAND_MAX >= 1
+            return (*func)(argbuf[0], argbuf[1], eval_state);
+#endif
+        case 4:
+#if SCM_FUNCTYPE_MAND_MAX >= 2
+            return (*func)(argbuf[0], argbuf[1], argbuf[2], eval_state);
+#endif
+        case 5:
+#if SCM_FUNCTYPE_MAND_MAX >= 3
+            return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], eval_state);
+#endif
+        case 6:
+#if SCM_FUNCTYPE_MAND_MAX >= 4
+            return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], argbuf[4], eval_state);
+#endif
+        case 7:
+#if SCM_FUNCTYPE_MAND_MAX >= 5
+            return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], argbuf[4], argbuf[5], eval_state);
+#endif
+        default:
+            SCM_NOTREACHED;
+        }
+    }
+#endif /* (SIZEOF_VOID_P < SIZEOF_SCMOBJ) */
+
     switch (i) {
     case 0:
         return (*func)();
     case 1:
-        if ((sizeof(void*) < sizeof(ScmObj)) && (type & SCM_FUNCTYPE_TAILREC))
-            return (*func)((void*)argbuf[0]);
         return (*func)(argbuf[0]);
     case 2:
-        if ((sizeof(void*) < sizeof(ScmObj)) && (type & SCM_FUNCTYPE_TAILREC))
-            return (*func)(argbuf[0], (void*)argbuf[0]);
         return (*func)(argbuf[0], argbuf[1]);
     case 3:
 #if SCM_FUNCTYPE_MAND_MAX >= 1
-        if ((sizeof(void*) < sizeof(ScmObj)) && (type & SCM_FUNCTYPE_TAILREC))
-            return (*func)(argbuf[0], argbuf[1], (void*)argbuf[0]);
         return (*func)(argbuf[0], argbuf[1], argbuf[2]);
 #endif
     case 4:
 #if SCM_FUNCTYPE_MAND_MAX >= 2
-        if ((sizeof(void*) < sizeof(ScmObj)) && (type & SCM_FUNCTYPE_TAILREC))
-            return (*func)(argbuf[0], argbuf[1], argbuf[2], (void*)argbuf[0]);
         return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3]);
 #endif
     case 5:
 #if SCM_FUNCTYPE_MAND_MAX >= 3
-        if ((sizeof(void*) < sizeof(ScmObj)) && (type & SCM_FUNCTYPE_TAILREC))
-            return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], (void*)argbuf[0]);
         return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], argbuf[4]);
 #endif
     case 6:
 #if SCM_FUNCTYPE_MAND_MAX >= 4
-        if ((sizeof(void*) < sizeof(ScmObj)) && (type & SCM_FUNCTYPE_TAILREC))
-            return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], argbuf[4], (void*)argbuf[0]);
         return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], argbuf[4], argbuf[5]);
 #endif
     case 7:
 #if SCM_FUNCTYPE_MAND_MAX >= 5
-        if ((sizeof(void*) < sizeof(ScmObj)) && (type & SCM_FUNCTYPE_TAILREC))
-            return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], argbuf[4], argbuf[5], (void*)argbuf[0]);
         return (*func)(argbuf[0], argbuf[1], argbuf[2], argbuf[3], argbuf[4], argbuf[5], argbuf[6]);
 #endif
 
     default:
-        SCM_ASSERT(scm_false);
-        return SCM_INVALID;
+        SCM_NOTREACHED;
     }
 }
 
 /*===========================================================================
   S-Expression Evaluation
 ===========================================================================*/
+/*
+ * FIXME: I'm not sure what we should do with 'eval' to conform to following
+ * specification. See also 'rec-by-eval' of test-tail-rec.scm.
+ *   -- YamaKen 2006-09-25
+ *
+ * R5RS: 3.5 Proper tail recursion
+ * > Certain built-in procedures are also required to perform tail calls. The
+ * > first argument passed to apply and to call-with-current-continuation, and
+ * > the second argument passed to call-with-values, must be called via a tail
+ * > call.  Similarly, eval must evaluate its argument as if it were in tail
+ * > position within the eval procedure.
+ */
 SCM_EXPORT ScmObj
 scm_p_eval(ScmObj obj, ScmObj env)
 {
@@ -388,32 +471,53 @@ scm_eval(ScmObj obj, ScmObj env)
 {
     ScmEvalState state;
 
-#if SCM_DEBUG
-    scm_push_trace_frame(obj, env);
-#endif
-
+#if SCM_STRICT_TOPLEVEL_DEFINITIONS
+    /* FIXME: temporary hack */
+    if (EQ(env, SCM_INTERACTION_ENV_INDEFINABLE)) {
+        env = SCM_INTERACTION_ENV;
+        SCM_EVAL_STATE_INIT1(state, env);
+        state.nest = SCM_NEST_COMMAND;
+    } else if (EQ(env, SCM_INTERACTION_ENV)) {
+        SCM_EVAL_STATE_INIT1(state, env);
+        state.nest = SCM_NEST_PROGRAM;
+    } else {
+        SCM_EVAL_STATE_INIT1(state, env);
+    }
+#else
     /* intentionally does not use SCM_EVAL_STATE_INIT() to avoid overhead */
     state.env = env;
+#endif
+
+#if SCM_USE_BACKTRACE
+    scm_push_trace_frame(obj, env);
+#endif
 
 eval_loop:
     if (IDENTIFIERP(obj)) {
         obj = scm_symbol_value(obj, state.env);
     } else if (CONSP(obj)) {
         obj = call(CAR(obj), CDR(obj), &state, SCM_VALTYPE_NEED_EVAL);
-        if (state.ret_type == SCM_VALTYPE_NEED_EVAL)
+        if (state.ret_type == SCM_VALTYPE_NEED_EVAL) {
+#if SCM_STRICT_TOPLEVEL_DEFINITIONS
+            if (state.nest == SCM_NEST_RETTYPE_BEGIN)
+                state.nest = SCM_NEST_COMMAND_OR_DEFINITION;
+            else
+                state.nest = SCM_NEST_COMMAND;
+#endif
             goto eval_loop;
+        }
     }
 #if SCM_STRICT_NULL_FORM
     /* () is allowed by default for efficiency */
     else if (NULLP(obj))
         PLAIN_ERR("eval: () is not a valid R5RS form. use '() instead");
 #endif
-#if SCM_STRICT_VECTOR_FORM
+#if (SCM_USE_VECTOR && SCM_STRICT_VECTOR_FORM)
     else if (VECTORP(obj))
         PLAIN_ERR("eval: #() is not a valid R5RS form. use '#() instead");
 #endif
 
-#if SCM_DEBUG
+#if SCM_USE_BACKTRACE
     scm_pop_trace_frame();
 #endif
     return obj;
@@ -441,6 +545,9 @@ scm_p_apply(ScmObj proc, ScmObj arg0, ScmObj rest, ScmEvalState *eval_state)
 
     ENSURE_LIST(last);
 
+    /* Since any tail recursive procedures called here return a tail expression
+     * with SCM_VALTYPE_NEED_EVAL, evaluate such proc with call() does not
+     * break proper tail recursion of 'apply'.  -- YamaKen 2006-09-25 */
     return call(proc, args, eval_state, SCM_VALTYPE_AS_IS);
 }
 
@@ -464,10 +571,7 @@ map_eval(ScmObj args, scm_int_t *args_len, ScmObj env)
     FOR_EACH_PAIR (rest, args) {
         len++;
         elm = EVAL(CAR(rest), env);
-#if SCM_STRICT_ARGCHECK
-        if (VALUEPACKETP(elm))
-            ERR_OBJ("multiple values are not allowed here", elm);
-#endif
+        CHECK_VALID_EVALED_VALUE(elm);
         SCM_QUEUE_ADD(q, elm);
     }
     if (!NULLP(rest))
@@ -493,7 +597,7 @@ scm_p_scheme_report_environment(ScmObj version)
     ERR(SCM_ERRMSG_NON_R5RS_ENV);
 #else
     CDBG((SCM_DBG_COMPAT,
-          "scheme-report-environment: warning:" SCM_ERRMSG_NON_R5RS_ENV));
+          "scheme-report-environment: warning: " SCM_ERRMSG_NON_R5RS_ENV));
 #endif
 
     return SCM_R5RS_ENV;
@@ -512,7 +616,7 @@ scm_p_null_environment(ScmObj version)
     ERR(SCM_ERRMSG_NON_R5RS_ENV);
 #else
     CDBG((SCM_DBG_COMPAT,
-          "null-environment: warning:" SCM_ERRMSG_NON_R5RS_ENV));
+          "null-environment: warning: " SCM_ERRMSG_NON_R5RS_ENV));
 #endif
 
     return SCM_NULL_ENV;
