@@ -3,6 +3,7 @@
  *  About    : R5RS hygienic macros
  *
  *  Copyright (C) 2006 Jun Inoue <jun.lambda@gmail.com>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -36,6 +37,8 @@
 #include "sigschemeinternal.h"
 #include <stdlib.h>
 
+#include "functable-r5rs-macro.c"
+
 /* ------------------------------
  *  Definitions
  * ------------------------------
@@ -107,14 +110,12 @@ SCM_GLOBAL_VARS_BEGIN(static_macro);
 #if SCM_DEBUG_MACRO
 static enum dbg_flag l_debug_mode;
 #endif
-static ScmObj sym_ellipsis;     /* Currently has no use outside this file. */
+static int l_dummy;
 #undef static
 SCM_GLOBAL_VARS_END(static_macro);
 #define l_debug_mode SCM_GLOBAL_VAR(static_macro, l_debug_mode)
-#define sym_ellipsis SCM_GLOBAL_VAR(static_macro, sym_ellipsis)
 SCM_DEFINE_STATIC_VARS(static_macro);
 
-#define SYM_ELLIPSIS     sym_ellipsis
 #define SYM_SYNTAX_RULES scm_intern("syntax-rules")
 
 #define ELLIPSISP(o) EQ((o), SYM_ELLIPSIS)
@@ -211,16 +212,10 @@ static scm_int_t list_find_index(ScmObj x, ScmObj ls);
 SCM_EXPORT void
 scm_init_macro(void)
 {
-    /* TODO: parameterize EVAL in scm_s_let(), scm_s_letrec() and
-     * scm_s_define(), and call them with EVAL set to
-     * eval_syntax_rules().  We might want to separate macros
-     * namespace from that of objects since macros aren't first-class
-     * objects. */
-    scm_define_alias("let-syntax", "let");
-    scm_define_alias("letrec-syntax", "letrec");
-    scm_define_alias("define-syntax", "define");
     SCM_GLOBAL_VARS_INIT(static_macro);
-    sym_ellipsis = scm_intern("...");
+
+    scm_register_funcs(scm_functable_r5rs_macro);
+
     INIT_DBG();
 }
 
@@ -374,6 +369,37 @@ scm_s_match(ScmObj form, ScmObj clauses, ScmEvalState *state)
     return SCM_UNDEF;
 }
 
+/* TODO: parameterize EVAL in scm_s_let(), scm_s_letrec() and scm_s_define(),
+ * and call them with EVAL set to eval_syntax_rules().  We might want to
+ * separate macros namespace from that of objects since macros aren't
+ * first-class objects. */
+SCM_EXPORT ScmObj
+scm_s_let_syntax(ScmObj bindings, ScmObj body, ScmEvalState *eval_state)
+{
+    DECLARE_FUNCTION("let-syntax", syntax_variadic_tailrec_1);
+
+    return scm_s_let_internal(ScmMacro, bindings, body, eval_state);
+}
+
+SCM_EXPORT ScmObj
+scm_s_letrec_syntax(ScmObj bindings, ScmObj body, ScmEvalState *eval_state)
+{
+    DECLARE_FUNCTION("letrec-syntax", syntax_variadic_tailrec_1);
+
+    return scm_s_letrec_internal(ScmMacro, bindings, body, eval_state);
+}
+
+SCM_EXPORT ScmObj
+scm_s_define_syntax(ScmObj var, ScmObj macro, ScmObj env)
+{
+    DECLARE_FUNCTION("define-syntax", syntax_fixed_2);
+
+    ENSURE_IDENTIFIER(var);
+
+    scm_s_define_internal(ScmMacro, var, macro, env);
+
+    return SCM_UNDEF;
+}
 
 /* ==============================
  * Pattern Compiler
@@ -880,7 +906,7 @@ transcribe_rec(transcription_context *ctx, ScmObj template, ScmObj sub,
             case MSG_SPLICE:                                    \
                 SCM_QUEUE_APPEND((_q), r.u.obj); break;         \
             default:                                            \
-                SCM_ASSERT(scm_false);                          \
+                SCM_NOTREACHED;                                 \
             }                                                   \
         } while (0)
 
