@@ -3,7 +3,7 @@
  *  About    : SRFI-34 Exception Handling for Programs
  *
  *  Copyright (C) 2005-2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
- *  Copyright (c) 2007 SigScheme Project <uim AT freedesktop.org>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -150,7 +150,7 @@ scm_initialize_srfi34(void)
 
     SCM_GLOBAL_VARS_INIT(static_srfi34);
 
-    scm_use("srfi-23");
+    scm_require_module("srfi-23");
 
     /* protect global variables: assumes that all ScmObj of the global vars
      * instance is aligned */
