@@ -5,7 +5,7 @@
  *  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
  *  Copyright (C) 2005-2006 Jun Inoue <jun.lambda AT gmail.com>
  *  Copyright (C) 2005-2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
- *  Copyright (c) 2007 SigScheme Project <uim AT freedesktop.org>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -46,8 +46,8 @@
 #include <stdarg.h>
 
 #include <gcroots.h>
-#if (GCROOTS_API_REVISION < 0)
-#error "libgcroots that has API revision >= 0 is required"
+#if (GCROOTS_API_REVISION < 1)
+#error "libgcroots that has API revision >= 1 is required"
 #endif
 
 #include "scmint.h"
@@ -65,9 +65,10 @@ extern "C" {
 /*=======================================
   Macro Definitions
 =======================================*/
+#define SSCM_VERSION_STRING     "0.8.0"
 #define SSCM_VERSION_MAJOR      (0)
-#define SSCM_VERSION_MINOR      (7)
-#define SSCM_VERSION_PATCHLEVEL (4)
+#define SSCM_VERSION_MINOR      (8)
+#define SSCM_VERSION_PATCHLEVEL (0)
 #define SSCM_VERSION_REQUIRE(major, minor, patchlevel)			     \
   ((major) < SSCM_VERSION_MAJOR						     \
    || ((major) == SSCM_VERSION_MAJOR && (minor) < SSCM_VERSION_MINOR)	     \
@@ -818,12 +819,25 @@ struct ScmStorageConf_ {
                             && !(SCM_FUNC_TYPECODE(o) & SCM_FUNCTYPE_SYNTAX)) \
                            || SCM_CLOSUREP(o)                                \
                            || SCM_CONTINUATIONP(o))
-#if SCM_USE_HYGIENIC_MACRO
+
+#if (SCM_USE_HYGIENIC_MACRO && SCM_USE_LEGACY_MACRO)
+#define SCM_SYNTACTIC_OBJECTP(o) (SCM_SYNTAXP(o) || SCM_HMACROP(o)      \
+                                  || SCM_SYNTACTIC_CLOSUREP(o))
+#elif SCM_USE_HYGIENIC_MACRO
 #define SCM_SYNTACTIC_OBJECTP(o) (SCM_SYNTAXP(o) || SCM_HMACROP(o))
+#elif SCM_USE_LEGACY_MACRO
+#define SCM_SYNTACTIC_OBJECTP(o) (SCM_SYNTAXP(o) || SCM_SYNTACTIC_CLOSUREP(o))
 #else
 #define SCM_SYNTACTIC_OBJECTP(o) (SCM_SYNTAXP(o))
 #endif
 
+#if SCM_USE_LEGACY_MACRO
+#define SCM_SYNTACTIC_CLOSURE_ENV scm_syntactic_closure_env
+#define SCM_SYNTACTIC_CLOSUREP(o)                                       \
+    (SCM_CLOSUREP(o)                                                    \
+     && SCM_EQ(SCM_CLOSURE_ENV(o), SCM_SYNTACTIC_CLOSURE_ENV))
+#endif
+
 #define SCM_CLOSUREP(o)                 SCM_SAL_CLOSUREP(o)
 #define SCM_CLOSURE_EXP(o)              SCM_SAL_CLOSURE_EXP(o)
 #define SCM_CLOSURE_ENV(o)              SCM_SAL_CLOSURE_ENV(o)
@@ -1009,6 +1023,27 @@ struct ScmStorageConf_ {
 #define SCM_SYM_UNQUOTE_SPLICING scm_sym_unquote_splicing
 #define SCM_SYM_ELLIPSIS         scm_sym_ellipsis
 
+/*=======================================
+  List Constructor
+=======================================*/
+typedef ScmRef ScmQueue;
+#define SCM_QUEUE_INVALIDATE(_q)     ((_q) = SCM_INVALID_REF)
+#define SCM_QUEUE_VALIDP(_q)         ((_q) != SCM_INVALID_REF)
+#define SCM_QUEUE_POINT_TO(_q, _out) ((_q) = SCM_REF_OFF_HEAP(_out))
+#define SCM_QUEUE_ADD(_q, _dat)      (SCM_SET((_q), SCM_LIST_1(_dat)),       \
+                                      (_q) = SCM_REF_CDR(SCM_DEREF(_q)))
+#define SCM_QUEUE_CONST_ADD(_q, _dat)                                        \
+    (SCM_SET((_q), SCM_IMMUTABLE_CONS((_dat), SCM_NULL)),                    \
+     (_q) = SCM_REF_CDR(SCM_DEREF(_q)))
+#define SCM_QUEUE_APPEND(_q, _lst)                                           \
+    do {                                                                     \
+        SCM_SET((_q), (_lst));                                               \
+        while (SCM_CONSP(SCM_DEREF(_q)))                                     \
+            (_q) = SCM_REF_CDR(SCM_DEREF(_q));                               \
+    } while (/* CONSTCOND */ 0)
+#define SCM_QUEUE_TERMINATOR(_q)          (SCM_DEREF(_q))
+#define SCM_QUEUE_SLOPPY_APPEND(_q, _lst) (SCM_SET((_q), (_lst)))
+
 /*=======================================
    Evaluator's State
 =======================================*/
@@ -1213,6 +1248,18 @@ SCM_GLOBAL_VARS_END(syntax);
 #define scm_sym_ellipsis         SCM_GLOBAL_VAR(syntax, scm_sym_ellipsis)
 SCM_DECLARE_EXPORTED_VARS(syntax);
 
+/* legacy-macro.c */
+#if SCM_USE_LEGACY_MACRO
+/* Don't use scm_syntactic_closure_env directly. Use SCM_SYNTACTIC_CLOSURE_ENV
+ * instead. */
+SCM_GLOBAL_VARS_BEGIN(legacy_macro);
+ScmObj scm_syntactic_closure_env;
+SCM_GLOBAL_VARS_END(legacy_macro);
+#define scm_syntactic_closure_env                                       \
+    SCM_GLOBAL_VAR(legacy_macro, scm_syntactic_closure_env)
+SCM_DECLARE_EXPORTED_VARS(legacy_macro);
+#endif /* SCM_USE_LEGACY_MACRO */
+
 /*=======================================
   Function Declarations
 =======================================*/
@@ -1220,17 +1267,26 @@ SCM_DECLARE_EXPORTED_VARS(syntax);
    SigScheme: Core Functions
 ===========================================================================*/
 /* sigscheme.c */
-SCM_EXPORT void scm_initialize(const ScmStorageConf *storage_conf);
+SCM_EXPORT char **scm_initialize(const ScmStorageConf *storage_conf,
+                                 const char *const *argv);
 SCM_EXPORT void scm_finalize(void);
 #if SCM_USE_EVAL_C_STRING
 SCM_EXPORT ScmObj scm_eval_c_string(const char *exp);
 #endif
+SCM_EXPORT ScmObj scm_array2list(void **ary, size_t len,
+                                 ScmObj (*conv)(void *));
+SCM_EXPORT void **scm_list2array(ScmObj lst, size_t *len,
+                                 void *(*conv)(ScmObj));
 
 /* module.c */
 SCM_EXPORT void scm_provide(ScmObj feature);
 SCM_EXPORT scm_bool scm_providedp(ScmObj feature);
+#if 1  /* 'use' is deprecated and will be removed in SigScheme 0.9 */
 SCM_EXPORT scm_bool scm_use(const char *feature);
 SCM_EXPORT ScmObj scm_s_use(ScmObj feature, ScmObj env);
+#endif
+SCM_EXPORT scm_bool scm_require_module(const char *name);
+SCM_EXPORT ScmObj scm_p_require_module(ScmObj name);
 SCM_EXPORT ScmObj scm_register_func(const char *name, ScmFuncType func,
                                     enum ScmFuncTypeCode type);
 SCM_EXPORT void scm_register_funcs(const struct scm_func_registration_info *table);
@@ -1247,7 +1303,12 @@ SCM_EXPORT char *scm_strdup(const char *str);
 SCM_EXPORT void scm_gc_protect(ScmObj *var);
 SCM_EXPORT void scm_gc_protect_with_init(ScmObj *var, ScmObj init_val);
 SCM_EXPORT void scm_gc_unprotect(ScmObj *var);
+SCM_EXPORT scm_bool scm_gc_protectedp(ScmObj obj);
 SCM_EXPORT void *scm_call_with_gc_ready_stack(ScmGCGateFunc func, void *arg);
+SCM_EXPORT scm_bool scm_gc_protected_contextp(void);
+/* for semantic assertions */
+#define scm_gc_any_contextp()                                                \
+    (!scm_gc_protected_contextp() || scm_gc_protected_contextp())
 
 /* symbol.c */
 SCM_EXPORT ScmObj scm_intern(const char *name);
@@ -1477,6 +1538,8 @@ SCM_EXPORT ScmObj scm_p_substring(ScmObj str, ScmObj start, ScmObj end);
 SCM_EXPORT ScmObj scm_p_string2list(ScmObj str);
 SCM_EXPORT ScmObj scm_p_list2string(ScmObj lst);
 SCM_EXPORT ScmObj scm_p_string_fillx(ScmObj str, ScmObj ch);
+SCM_EXPORT ScmObj scm_p_string_mutablep(ScmObj str);
+SCM_EXPORT ScmObj scm_p_string_reconstructx(ScmObj str);
 #endif /* SCM_USE_STRING_PROCEDURE */
 
 /* vector.c */
@@ -1490,6 +1553,7 @@ SCM_EXPORT ScmObj scm_p_vector_setx(ScmObj vec, ScmObj _k, ScmObj obj);
 SCM_EXPORT ScmObj scm_p_vector2list(ScmObj vec);
 SCM_EXPORT ScmObj scm_p_list2vector(ScmObj lst);
 SCM_EXPORT ScmObj scm_p_vector_fillx(ScmObj vec, ScmObj fill);
+SCM_EXPORT ScmObj scm_p_vector_mutablep(ScmObj vec);
 #endif /* SCM_USE_VECTOR */
 
 /* deep-cadrs.c */
@@ -1548,14 +1612,14 @@ SCM_EXPORT void scm_port_puts(ScmObj port, const char *str);
 SCM_EXPORT void scm_port_put_char(ScmObj port, scm_ichar_t ch);
 SCM_EXPORT void scm_port_newline(ScmObj port);
 SCM_EXPORT void scm_port_flush(ScmObj port);
-SCM_EXPORT ScmObj scm_p_call_with_input_file(ScmObj filepath, ScmObj proc);
-SCM_EXPORT ScmObj scm_p_call_with_output_file(ScmObj filepath, ScmObj proc);
 SCM_EXPORT ScmObj scm_p_input_portp(ScmObj obj);
 SCM_EXPORT ScmObj scm_p_output_portp(ScmObj obj);
 SCM_EXPORT ScmObj scm_p_current_input_port(void);
 SCM_EXPORT ScmObj scm_p_current_output_port(void);
-SCM_EXPORT ScmObj scm_p_with_input_from_file(ScmObj filepath, ScmObj thunk);
-SCM_EXPORT ScmObj scm_p_with_output_to_file(ScmObj filepath, ScmObj thunk);
+SCM_EXPORT ScmObj scm_p_current_error_port(void);
+SCM_EXPORT ScmObj scm_p_set_current_input_portx(ScmObj newport);
+SCM_EXPORT ScmObj scm_p_set_current_output_portx(ScmObj newport);
+SCM_EXPORT ScmObj scm_p_set_current_error_portx(ScmObj newport);
 SCM_EXPORT ScmObj scm_p_open_input_file(ScmObj filepath);
 SCM_EXPORT ScmObj scm_p_open_output_file(ScmObj filepath);
 SCM_EXPORT ScmObj scm_p_close_input_port(ScmObj port);
@@ -1592,8 +1656,10 @@ SCM_EXPORT ScmObj scm_p_display(ScmObj obj, ScmObj args);
 /* load.c */
 #if SCM_USE_LOAD
 SCM_EXPORT void scm_set_lib_path(const char *path);
+SCM_EXPORT void scm_set_system_load_path(const char *path);
 SCM_EXPORT void scm_load(const char *filename);
 SCM_EXPORT ScmObj scm_p_load_path(void);
+SCM_EXPORT ScmObj scm_p_system_load_path(void);
 SCM_EXPORT ScmObj scm_p_load(ScmObj filename);
 #endif /* SCM_USE_LOAD */
 
@@ -1610,12 +1676,22 @@ SCM_EXPORT ScmObj scm_format(ScmObj port, enum ScmFormatCapability fcap,
 /*===========================================================================
    SigScheme: Optional Funtions
 ===========================================================================*/
+/* legacy-macro.c */
+#if SCM_USE_LEGACY_MACRO
+SCM_EXPORT ScmObj scm_s_define_macro(ScmObj identifier, ScmObj rest,
+                                     ScmEvalState *eval_state);
+#endif /* SCM_USE_LEGACY_MACRO */
+
 /* module-sscm-ext.c */
 #if SCM_USE_SSCM_EXTENSIONS
 SCM_EXPORT void scm_require(const char *filename);
 SCM_EXPORT ScmObj scm_p_symbol_boundp(ScmObj sym, ScmObj rest);
+SCM_EXPORT ScmObj scm_p_sscm_version(void);
 SCM_EXPORT ScmObj scm_p_current_environment(ScmEvalState *eval_state);
+SCM_EXPORT ScmObj scm_p_current_char_codec(void);
+SCM_EXPORT ScmObj scm_p_set_current_char_codecx(ScmObj encoding);
 SCM_EXPORT ScmObj scm_p_prealloc_heaps(ScmObj n);
+SCM_EXPORT ScmObj scm_p_pair_mutablep(ScmObj kons);
 SCM_EXPORT ScmObj scm_p_fixnum_width(void);
 SCM_EXPORT ScmObj scm_p_least_fixnum(void);
 SCM_EXPORT ScmObj scm_p_greatest_fixnum(void);
@@ -1624,6 +1700,9 @@ SCM_EXPORT ScmObj scm_p_provide(ScmObj feature);
 SCM_EXPORT ScmObj scm_p_providedp(ScmObj feature);
 SCM_EXPORT ScmObj scm_p_lengthstar(ScmObj lst);
 SCM_EXPORT ScmObj scm_p_exit(ScmObj args) SCM_NORETURN;
+SCM_EXPORT ScmObj scm_s_let_optionalsstar(ScmObj args,
+                                          ScmObj bindings, ScmObj body,
+                                          ScmEvalState *eval_state);
 #endif /* SCM_USE_SSCM_EXTENSIONS */
 
 /* module-siod.c */
@@ -1641,42 +1720,12 @@ SCM_EXPORT void   scm_set_verbose_level(long level);
 
 /* module-srfi1.c */
 #if SCM_USE_SRFI1
-SCM_EXPORT ScmObj scm_p_srfi1_xcons(ScmObj a, ScmObj b);
-SCM_EXPORT ScmObj scm_p_srfi1_consstar(ScmObj args);
-SCM_EXPORT ScmObj scm_p_srfi1_make_list(ScmObj length, ScmObj args);
-SCM_EXPORT ScmObj scm_p_srfi1_list_tabulate(ScmObj _n, ScmObj args);
-SCM_EXPORT ScmObj scm_p_srfi1_list_copy(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_circular_list(ScmObj args);
-SCM_EXPORT ScmObj scm_p_srfi1_iota(ScmObj scm_count, ScmObj args);
-SCM_EXPORT ScmObj scm_p_srfi1_proper_listp(ScmObj obj);
 SCM_EXPORT ScmObj scm_p_srfi1_circular_listp(ScmObj obj);
 SCM_EXPORT ScmObj scm_p_srfi1_dotted_listp(ScmObj obj);
-SCM_EXPORT ScmObj scm_p_srfi1_not_pairp(ScmObj obj);
-SCM_EXPORT ScmObj scm_p_srfi1_null_listp(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_listequal(ScmObj eqproc, ScmObj args);
-SCM_EXPORT ScmObj scm_p_srfi1_first(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_second(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_third(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_fourth(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_fifth(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_sixth(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_seventh(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_eighth(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_ninth(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_tenth(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_carpluscdr(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_take(ScmObj lst, ScmObj scm_idx);
-SCM_EXPORT ScmObj scm_p_srfi1_drop(ScmObj lst, ScmObj scm_idx);
-SCM_EXPORT ScmObj scm_p_srfi1_take_right(ScmObj lst, ScmObj scm_elem);
-SCM_EXPORT ScmObj scm_p_srfi1_drop_right(ScmObj lst, ScmObj scm_elem);
-SCM_EXPORT ScmObj scm_p_srfi1_takex(ScmObj lst, ScmObj scm_idx);
-SCM_EXPORT ScmObj scm_p_srfi1_drop_rightx(ScmObj lst, ScmObj scm_idx);
-SCM_EXPORT ScmObj scm_p_srfi1_split_at(ScmObj lst, ScmObj idx);
-SCM_EXPORT ScmObj scm_p_srfi1_split_atx(ScmObj lst, ScmObj idx);
-SCM_EXPORT ScmObj scm_p_srfi1_last(ScmObj lst);
 SCM_EXPORT ScmObj scm_p_srfi1_last_pair(ScmObj lst);
 SCM_EXPORT ScmObj scm_p_srfi1_lengthplus(ScmObj lst);
-SCM_EXPORT ScmObj scm_p_srfi1_concatenate(ScmObj args);
+SCM_EXPORT ScmObj scm_p_srfi1_map_in_order(ScmObj proc, ScmObj args);
+SCM_EXPORT ScmObj scm_p_srfi1_find_tail(ScmObj pred, ScmObj lst);
 #endif /* SCM_USE_SRFI1 */
 
 /* module-srfi2.c */
@@ -1698,6 +1747,15 @@ SCM_EXPORT ScmObj scm_s_srfi8_receive(ScmObj formals, ScmObj expr, ScmObj body,
                                       ScmEvalState *eval_state);
 #endif
 
+/* module-srfi9.c */
+#if SCM_USE_SRFI9
+SCM_EXPORT ScmObj scm_s_srfi9_define_record_type(ScmObj type_name,
+                                                 ScmObj ctor_spec,
+                                                 ScmObj pred_name,
+                                                 ScmObj field_specs,
+                                                 ScmEvalState *eval_state);
+#endif
+
 /* module-srfi23.c */
 #if SCM_USE_SRFI23
 SCM_EXPORT ScmObj scm_p_srfi23_error(ScmObj reason, ScmObj args);
@@ -1729,6 +1787,11 @@ SCM_EXPORT ScmObj scm_p_srfi48_format(ScmObj fmt_or_port, ScmObj rest);
 SCM_EXPORT ScmObj scm_p_formatplus(ScmObj fmt_or_port, ScmObj rest);
 #endif
 
+/* module-srfi55.c */
+#if SCM_USE_SRFI55
+SCM_EXPORT ScmObj scm_s_srfi55_require_extension(ScmObj clauses, ScmObj env);
+#endif
+
 /* module-srfi60.c */
 #if SCM_USE_SRFI60
 SCM_EXPORT ScmObj scm_p_srfi60_logand(ScmObj left, ScmObj right,
