@@ -3,6 +3,9 @@
  *  About    : variable and function definitions for internal use
  *
  *  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
+ *  Copyright (C) 2005-2006 Jun Inoue <jun.lambda AT gmail.com>
+ *  Copyright (C) 2005-2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -41,7 +44,11 @@
 
 #include "global.h"
 #include "sigscheme.h"
+#if SCM_USE_MULTIBYTE_CHAR
 #include "encoding.h"
+#else
+#include "encoding-dummy.h"
+#endif
 #if SCM_USE_PORT
 #include "scmport.h"
 #endif
@@ -55,10 +62,12 @@ extern "C" {
 =======================================*/
 /* TODO: generate these automatically and maybe put them in an optional public
  * header file. */
+
 #define SYM_QUOTE            SCM_SYM_QUOTE
 #define SYM_QUASIQUOTE       SCM_SYM_QUASIQUOTE
 #define SYM_UNQUOTE          SCM_SYM_UNQUOTE
 #define SYM_UNQUOTE_SPLICING SCM_SYM_UNQUOTE_SPLICING
+#define SYM_ELLIPSIS         SCM_SYM_ELLIPSIS
 
 #define EQ             SCM_EQ
 #define NULLP          SCM_NULLP
@@ -111,17 +120,13 @@ extern "C" {
 #define MAKE_IMMUTABLE_VECTOR         SCM_MAKE_IMMUTABLE_VECTOR
 #define MAKE_PORT                     SCM_MAKE_PORT
 #define MAKE_CONTINUATION             SCM_MAKE_CONTINUATION
-#if SCM_USE_SSCM_EXTENSIONS
 #define MAKE_C_POINTER                SCM_MAKE_C_POINTER
 #define MAKE_C_FUNCPOINTER            SCM_MAKE_C_FUNCPOINTER
-#endif /* SCM_USE_SSCM_EXTENSIONS */
 #define MAKE_VALUEPACKET              SCM_MAKE_VALUEPACKET
 
-#if SCM_USE_HYGIENIC_MACRO
 #define MAKE_HMACRO                   SCM_MAKE_HMACRO
 #define MAKE_FARSYMBOL                SCM_MAKE_FARSYMBOL
 #define MAKE_SUBPAT                   SCM_MAKE_SUBPAT
-#endif /* SCM_USE_HYGIENIC_MACRO */
 
 #define NUMBERP        SCM_NUMBERP
 #define INTP           SCM_INTP
@@ -132,19 +137,17 @@ extern "C" {
 #define FUNCP          SCM_FUNCP
 #define SYNTAXP        SCM_SYNTAXP
 #define CLOSUREP       SCM_CLOSUREP
+#define SYNTACTIC_CLOSUREP SCM_SYNTACTIC_CLOSUREP
 #define PROCEDUREP     SCM_PROCEDUREP
+#define SYNTACTIC_OBJECTP SCM_SYNTACTIC_OBJECTP
 #define VECTORP        SCM_VECTORP
 #define PORTP          SCM_PORTP
 #define CONTINUATIONP  SCM_CONTINUATIONP
-#if SCM_USE_VALUECONS
 #define NULLVALUESP    SCM_NULLVALUESP
-#endif /* SCM_USE_VALUECONS */
 #define VALUEPACKETP   SCM_VALUEPACKETP
-#if SCM_USE_HYGIENIC_MACRO
 #define HMACROP        SCM_HMACROP
 #define FARSYMBOLP     SCM_FARSYMBOLP
 #define SUBPATP        SCM_SUBPATP
-#endif /* SCM_USE_HYGIENIC_MACRO */
 #define FREECELLP      SCM_FREECELLP
 #define C_POINTERP     SCM_C_POINTERP
 #define C_FUNCPOINTERP SCM_C_FUNCPOINTERP
@@ -186,14 +189,27 @@ extern "C" {
 /*=======================================
   Macro Definitions
 =======================================*/
-#define SCM_ERR_HEADER "ERROR: "
+#define SCM_ERR_HEADER "Error: "
 
-#define SCM_ERRMSG_UNHANDLED_EXCEPTION "unhandled exception"
+#define ERRMSG_FIXNUM_OVERFLOW     "fixnum overflow"
+#define ERRMSG_UNHANDLED_EXCEPTION "unhandled exception"
 #define SCM_ERRMSG_IMPROPER_ARGS                                             \
     "proper list required for function call but got"
 #define SCM_ERRMSG_NULL_IN_STRING                                            \
     "null character in a middle of string is not enabled"
+#define ERRMSG_UNSUPPORTED_ENCODING "unsupported encoding"
+#define ERRMSG_CODEC_SW_NOT_SUPPORTED                                        \
+    "character encoding switching is not supported on this build"
 
+#if SCM_STRICT_TOPLEVEL_DEFINITIONS
+/* FIXME: temporary hack. SCM_EOF is only used as an unique ID. */
+#define SCM_INTERACTION_ENV_INDEFINABLE SCM_EOF
+#endif
+
+/* specifies whether the storage abstraction layer can only handle nested
+ * (stacked) continuation or R5RS-conformant full implementation. But current
+ * implementation only supports '1'. */
+#define SCM_NESTED_CONTINUATION_ONLY 1
 #define INVALID_CONTINUATION_OPAQUE  NULL
 
 /* trace stack for debugging */
@@ -201,6 +217,7 @@ extern "C" {
 #define TRACE_FRAME_OBJ CAR
 #define TRACE_FRAME_ENV CDR
 
+/* TODO: Remove valuecons to increase simiplicity. */
 /* Extraction of a valuepacket is granted only for SigScheme-internals */
 #define SCM_VALUEPACKET_VALUES(o)    SCM_SAL_VALUEPACKET_VALUES(o)
 #if SCM_USE_VALUECONS
@@ -212,6 +229,8 @@ extern "C" {
     SCM_SAL_VALUEPACKET_SET_VALUES((o), (vals))
 #endif /* SCM_USE_VALUECONS */
 
+/* TODO: Remove the concept 'freecell object' from SAL and replace with
+ * ordinary cons cells with freecell-marker in storage-{compact,fatty}. */
 #define SCM_AS_FREECELL(o)              SCM_SAL_AS_FREECELL(o)
 
 #define SCM_FREECELLP(o)                SCM_SAL_FREECELLP(o)
@@ -221,24 +240,7 @@ extern "C" {
 #define SCM_FREECELL_SET_FREESLOT(o, v) SCM_SAL_FREECELL_SET_FREESLOT((o), (v))
 #define SCM_FREECELL_CLEAR_FREESLOT(o)  SCM_SAL_FREECELL_CLEAR_FREESLOT((o))
 
-/* For optimized operation: Cleanup a destructed ScmCell *cell to a freecell
- * and chain it into freelist. */
-#define SCM_RECLAIM_CELL(cell, next)    SCM_SAL_RECLAIM_CELL((cell), (next))
-
-#if 0
-/* for future cleanup */
-#define SCM_CELL_MARKEDP(cell)   SCM_SAL_CELL_MARKEDP(cell)
-#define SCM_CELL_UNMARKEDP(cell) SCM_SAL_CELL_UNMARKEDP(cell)
-#define SCM_CELL_MARK(cell)      SCM_SAL_CELL_MARK(cell)
-#define SCM_CELL_UNMARK(cell)    SCM_SAL_CELL_UNMARK(cell)
-#else
-#define SCM_MARKEDP(o)   SCM_SAL_MARKEDP(o)
-#define SCM_UNMARKEDP(o) SCM_SAL_UNMARKEDP(o)
-#define SCM_MARK(o)      SCM_SAL_MARK(o)
-#define SCM_UNMARK(o)    SCM_SAL_UNMARK(o)
-#endif
-
-#define EQVP(a, b)   (TRUEP(scm_p_eqvp((a), (b))))
+#define EQVP(a, b)   (SCM_EQVP((a), (b)))
 #define EQUALP(a, b) (TRUEP(scm_p_equalp((a), (b))))
 #define STRING_EQUALP(str1, str2)                                            \
     (EQ((str1), (str2))                                                      \
@@ -246,7 +248,9 @@ extern "C" {
          && strcmp(SCM_STRING_STR(str1), SCM_STRING_STR(str2)) == 0))
 
 /* result encoders for scm_length() */
-#define SCM_LISTLEN_ENCODE_DOTTED(len)   (-(len))
+/* Dotted list length (follows SRFI-1 definition) is encoded as
+ * (-length - 1) */
+#define SCM_LISTLEN_ENCODE_DOTTED(len)   (~(len))  /* (-len - 1) */
 #define SCM_LISTLEN_ENCODE_CIRCULAR(len) (SCM_INT_T_MIN)
 #define SCM_LISTLEN_ENCODE_ERROR         SCM_LISTLEN_ENCODE_CIRCULAR
 
@@ -308,6 +312,15 @@ SCM_EXPORT void scm_error_with_implicit_func(const char *msg, ...) SCM_NORETURN;
  * the error port. */
 #define ERR_OBJ(msg, obj) scm_error_obj(SCM_MANGLE(name), (msg), (obj))
 
+#define SCM_ENSURE_PROPER_LIST_TERMINATION(term, lst)                        \
+    (NULLP(term) || (ERR_OBJ("proper list required but got", (lst)), 1))
+
+#if SCM_STRICT_ARGCHECK
+#define SCM_CHECK_PROPER_LIST_TERMINATION SCM_ENSURE_PROPER_LIST_TERMINATION
+#else
+#define SCM_CHECK_PROPER_LIST_TERMINATION(term, lst) SCM_EMPTY_EXPR
+#endif
+
 /* ASSERT_NO_MORE_ARG() asserts that the variadic argument list has
  * been exhausted.  The assertion is implicit in NO_MORE_ARG(), so
  * usually you don't have to call it explicitly.
@@ -362,31 +375,31 @@ SCM_EXPORT void scm_error_with_implicit_func(const char *msg, ...) SCM_NORETURN;
     SCM_ASSERT(CONSP(_lst));                                                 \
     while ((_elm) = POP(_lst), CONSP(_lst))
 
-#define ENSURE_TYPE(pred, typename, obj)                                     \
-    (pred(obj) || (ERR_OBJ(typename " required but got", (obj)), 1))
+#define ENSURE_TYPE(pred, _typename, obj)                                    \
+    (pred(obj) || (ERR_OBJ(_typename " required but got", (obj)), 1))
 
-#define ENSURE_INT(obj)     ENSURE_TYPE(INTP, "integer", (obj))
-#define ENSURE_CONS(obj)    ENSURE_TYPE(CONSP, "pair", (obj))
-#define ENSURE_SYMBOL(obj)  ENSURE_TYPE(SYMBOLP, "symbol", (obj))
-#define ENSURE_CHAR(obj)    ENSURE_TYPE(CHARP, "character", (obj))
-#define ENSURE_STRING(obj)  ENSURE_TYPE(STRINGP, "string", (obj))
-#define ENSURE_FUNC(obj)    ENSURE_TYPE(FUNCP, "function", (obj))
-#define ENSURE_CLOSURE(obj) ENSURE_TYPE(CLOSUREP, "closure", (obj))
-#define ENSURE_VECTOR(obj)  ENSURE_TYPE(VECTORP, "vector", (obj))
-#define ENSURE_PORT(obj)    ENSURE_TYPE(PORTP, "port", (obj))
-#define ENSURE_CONTINUATION(obj) ENSURE_TYPE(CONTINUATIONP, "continuation", (obj))
-#define ENSURE_PROCEDURE(obj) ENSURE_TYPE(PROCEDUREP, "procedure", (obj))
-#define ENSURE_ENV(obj)     ENSURE_TYPE(ENVP, "environment specifier", (obj))
-#define ENSURE_VALID_ENV(obj)                                                \
-    ENSURE_TYPE(VALID_ENVP, "valid environment specifier", (obj))
-#define ENSURE_ERROBJ(obj)  ENSURE_TYPE(ERROBJP, "error object", (obj))
-#define ENSURE_LIST(obj)    ENSURE_TYPE(LISTP, "list", (obj))
-#define ENSURE_IDENTIFIER(obj) ENSURE_TYPE(IDENTIFIERP, "identifier", (obj))
+#define ENSURE_INT(o)          ENSURE_TYPE(INTP,          "integer",      (o))
+#define ENSURE_CONS(o)         ENSURE_TYPE(CONSP,         "pair",         (o))
+#define ENSURE_SYMBOL(o)       ENSURE_TYPE(SYMBOLP,       "symbol",       (o))
+#define ENSURE_CHAR(o)         ENSURE_TYPE(CHARP,         "character",    (o))
+#define ENSURE_STRING(o)       ENSURE_TYPE(STRINGP,       "string",       (o))
+#define ENSURE_FUNC(o)         ENSURE_TYPE(FUNCP,         "function",     (o))
+#define ENSURE_CLOSURE(o)      ENSURE_TYPE(CLOSUREP,      "closure",      (o))
+#define ENSURE_VECTOR(o)       ENSURE_TYPE(VECTORP,       "vector",       (o))
+#define ENSURE_PORT(o)         ENSURE_TYPE(PORTP,         "port",         (o))
+#define ENSURE_CONTINUATION(o) ENSURE_TYPE(CONTINUATIONP, "continuation", (o))
+#define ENSURE_PROCEDURE(o)    ENSURE_TYPE(PROCEDUREP,    "procedure",    (o))
+#define ENSURE_ENV(o)          ENSURE_TYPE(ENVP, "environment specifier", (o))
+#define ENSURE_VALID_ENV(o)                                                \
+    ENSURE_TYPE(VALID_ENVP, "valid environment specifier", (o))
+#define ENSURE_ERROBJ(o)       ENSURE_TYPE(ERROBJP,       "error object", (o))
+#define ENSURE_LIST(o)         ENSURE_TYPE(LISTP,         "list",         (o))
+#define ENSURE_IDENTIFIER(o)   ENSURE_TYPE(IDENTIFIERP,   "identifier",   (o))
 
 #if SCM_HAS_IMMUTABLE_CONS
 #define ENSURE_MUTABLE_CONS(kons)                                            \
     (SCM_CONS_MUTABLEP(kons)                                                 \
-     || (ERR_OBJ("attempted to modify immutable pair", kons), 1))
+     || (ERR_OBJ("attempted to modify immutable pair", (kons)), 1))
 #else /* SCM_HAS_IMMUTABLE_CONS */
 #define ENSURE_MUTABLE_CONS(kons) SCM_EMPTY_EXPR
 #endif /* SCM_HAS_IMMUTABLE_CONS */
@@ -394,7 +407,7 @@ SCM_EXPORT void scm_error_with_implicit_func(const char *msg, ...) SCM_NORETURN;
 #if SCM_HAS_IMMUTABLE_STRING
 #define ENSURE_MUTABLE_STRING(str)                                           \
     (SCM_STRING_MUTABLEP(str)                                                \
-     || (ERR_OBJ("attempted to modify immutable string", str), 1))
+     || (ERR_OBJ("attempted to modify immutable string", (str)), 1))
 #else /* SCM_HAS_IMMUTABLE_STRING */
 #define ENSURE_MUTABLE_STRING(str) SCM_EMPTY_EXPR
 #endif /* SCM_HAS_IMMUTABLE_STRING */
@@ -402,7 +415,7 @@ SCM_EXPORT void scm_error_with_implicit_func(const char *msg, ...) SCM_NORETURN;
 #if SCM_HAS_IMMUTABLE_VECTOR
 #define ENSURE_MUTABLE_VECTOR(vec)                                           \
     (SCM_VECTOR_MUTABLEP(vec)                                                \
-     || (ERR_OBJ("attempted to modify immutable vector", vec), 1))
+     || (ERR_OBJ("attempted to modify immutable vector", (vec)), 1))
 #else /* SCM_HAS_IMMUTABLE_VECTOR */
 #define ENSURE_MUTABLE_VECTOR(vec) SCM_EMPTY_EXPR
 #endif /* SCM_HAS_IMMUTABLE_VECTOR */
@@ -418,24 +431,63 @@ SCM_EXPORT void scm_error_with_implicit_func(const char *msg, ...) SCM_NORETURN;
              SCM_CHARCODEC_ENCODING(codec)), 0))
 #endif /* SCM_USE_MULTIBYTE_CHAR */
 
+#if SCM_STRICT_ARGCHECK
+#define CHECK_VALID_EVALED_VALUE(x)                                          \
+    do {                                                                     \
+        if (SYNTACTIC_OBJECTP(x))                                            \
+            ERR_OBJ("syntactic keyword is evaluated as value", x);           \
+        if (VALUEPACKETP(x))                                                 \
+            ERR_OBJ("multiple values are not allowed here", x);              \
+    } while (/* CONSTCOND */ 0)
+#else
+#define CHECK_VALID_EVALED_VALUE(x) SCM_EMPTY_EXPR
+#endif
+
+/*=======================================
+  Numbers
+=======================================*/
+#define INT_VALID_VALUEP(i)  (SCM_INT_MIN <= (i) && (i) <= SCM_INT_MAX)
+#define INT_OUT_OF_RANGEP(i) (!INT_VALID_VALUEP(i))
+
 /*=======================================
   Characters
 =======================================*/
-/* accepts EOF */
-#define ICHAR_ASCIIP(c)      (0 <= (c) && (c) <= 127)
+/* FIXME: support R6RS Unicode */
+
+/* accepts SCM_ICHAR_EOF */
+/* assumes ASCII */
+#define ICHAR_ASCIIP(c)         (0 <= (c) && (c) <= 127)
+#define ICHAR_SINGLEBYTEP(c)    (0 <= (c) && (c) <= 255)
+#define ICHAR_VALID_UNICODEP(c) ((0 <= (c) && (c) <= 0xd7ff)                  \
+                                 || (0xe000 <= (c) && (c) <= 0x10ffff))
+
 #define ICHAR_CONTROLP(c)    ((0 <= (c) && (c) <= 31) || (c) == 127)
+/*
+ * SigScheme treats vertical tab (0x0b) as a white space charcter although
+ * R5RS char-whitespace? does not cover it.
+ *
+ * R5RS: 6.3.4 Characters
+ *   The whitespace characters are space, tab, line feed, form feed, and
+ *   carriage return.
+ *
+ * R6RS Standard Libraries: 1.1  Characters
+ *   A character is whitespace if it is in one of the space, line, or
+ *   paragraph separator categories (Zs, Zl or Zp), or if is U+0009
+ *   (Horizontal tabulation), U+000A (Line feed), U+000B (Vertical
+ *   tabulation), U+000C (Form feed), or U+000D (Carriage return).
+ */
 #define ICHAR_WHITESPACEP(c) ((c) == ' ' || ('\t' <= (c) && (c) <= '\r'))
 #define ICHAR_NUMERICP(c)    ('0' <= (c) && (c) <= '9')
 #define ICHAR_HEXA_NUMERICP(c) (ICHAR_NUMERICP(c)                            \
                                 || ('a' <= (c) && (c) <= 'f')                \
                                 || ('A' <= (c) && (c) <= 'F'))
-#define ICHAR_ALPHABETICP(c) (ICHAR_UPPER_CASEP(c) || ICHAR_LOWER_CASEP(c))
+#define ICHAR_ALPHABETICP(c) (ICHAR_LOWER_CASEP(c) || ICHAR_UPPER_CASEP(c))
 #define ICHAR_UPPER_CASEP(c) ('A' <= (c) && (c) <= 'Z')
 #define ICHAR_LOWER_CASEP(c) ('a' <= (c) && (c) <= 'z')
 
 /*
  * SigScheme's case-insensitive character comparison conforms to the
- * foldcase'ed comparison described in SRFI-75 and SRFI-13, although R5RS does
+ * foldcase'ed comparison described in R6RS and SRFI-13, although R5RS does
  * not define comparison between alphabetic and non-alphabetic char.
  *
  * This specification is needed to produce natural result on sort functions
@@ -448,207 +500,21 @@ SCM_EXPORT void scm_error_with_implicit_func(const char *msg, ...) SCM_NORETURN;
  *
  * See also:
  *
- *   - Description around 'char-foldcase' in SRFI-75
+ *   - Description around 'char-foldcase' in R6RS (R5.92) Standard Libraries
+ *     http://www.r6rs.org/document/lib-html/r6rs-lib-Z-H-3.html#node_sec_1.1
  *   - "Case mapping and case-folding" and "Comparison" section of SRFI-13
  */
-/* FIXME: support SRFI-75 */
 #define ICHAR_DOWNCASE(c) (ICHAR_UPPER_CASEP(c) ? (c) + ('a' - 'A') : (c))
 #define ICHAR_UPCASE(c)   (ICHAR_LOWER_CASEP(c) ? (c) - ('a' - 'A') : (c))
 /* foldcase for case-insensitive character comparison is done by downcase as
- * described in SRFI-75. Although SRFI-13 expects (char-downcase (char-upcase
+ * described in R6RS libs. Although SRFI-13 expects (char-downcase (char-upcase
  * c)), this implementation is sufficient for ASCII range. */
 #define ICHAR_FOLDCASE(c) (ICHAR_DOWNCASE(c))
 
-/*=======================================
-  List Constructor
-=======================================*/
-typedef ScmRef ScmQueue;
-#define SCM_QUEUE_INVALIDATE(_q) ((_q) = NULL)
-#define SCM_QUEUE_VALIDP(_q)     (_q)
-#define SCM_QUEUE_POINT_TO(_q, _out) ((_q) = SCM_REF_OFF_HEAP(_out))
-#define SCM_QUEUE_ADD(_q, _dat) (SET((_q), LIST_1(_dat)),                    \
-                                 (_q) = REF_CDR(DEREF(_q)))
-#define SCM_QUEUE_CONST_ADD(_q, _dat)                                        \
-    (SET((_q), IMMUTABLE_CONS((_dat), SCM_NULL)),                            \
-     (_q) = REF_CDR(DEREF(_q)))
-#define SCM_QUEUE_APPEND(_q, _lst)                                           \
-    do {                                                                     \
-        SET((_q), (_lst));                                                   \
-        while (CONSP(DEREF(_q)))                                             \
-            (_q) = REF_CDR(DEREF(_q));                                       \
-    } while (/* CONSTCOND */ 0)
-#define SCM_QUEUE_TERMINATOR(_q)          (DEREF(_q))
-#define SCM_QUEUE_SLOPPY_APPEND(_q, _lst) (SET((_q), (_lst)))
-
-/*=======================================
-   Sequential Datum Translator
-=======================================*/
-/*
- * These utilities are for copying a sequence with partial
- * modifications.  They're used for handling quasiquotation and macro
- * expansion.  Translator works as a copy-on-write iterator for a list
- * or vector.
- *
- * First, initialize the proper type of translator with either
- * TRL_INIT() or TRV_INIT(), supplying the datum to be duplicated.
- * Then, traverse over the `copy' by successively and alternately
- * calling TR_GET_ELM() and TR_NEXT().  If an item returned by
- * TR_GET_ELM() should be replaced, then call TR_EXECUTE() with the
- * message TR_REPLACE or TR_SPLICE (see their definition for details).
- * When TR_ENDP() returns true, stop and obtain the duplicate with
- * TR_EXTRACT().  TR_CALL() is a low-level construct that doesn't
- * demultiplex the return value.  Usually you would want TR_EXECUTE()
- * instead.  The only exception is if you expect a boolean to be
- * returned (those that test true for TR_BOOL_MSG_P()).
- *
- * The last cdr of an improper list is *not* considered a part of the
- * list and will be treated just like the () of a proper list.  In
- * order to retrieve that last cdr, call TRL_GET_SUBLS() *after*
- * TR_ENDP() returns true.  Replacement of that portion must be done
- * with TRL_SET_SUBLS().
- *
- * No operation except TRL_GET_SUBLS(), TRL_SET_SUBLS(), TR_EXTRACT(),
- * and TR_ENDP() can be done on a translator for which TR_ENDP()
- * returns true.
- *
- * Everything prefixed with TRL_ is specific to list translators.
- * Likewise, TRV_ shows specificity to vector translators.  TR_
- * denotes a polymorphism.
- */
-
-/**
- * Message IDs.  We have to bring this upfront because ISO C forbids
- * forward reference to enumerations.
- */
-enum _tr_msg {
-    /** Don't do anything. */
-    TR_MSG_NOP,
-
-    /** Put OBJ in place of the current element. */
-    TR_MSG_REPLACE,
-
-    /** Splice OBJ into the current cell. */
-    TR_MSG_SPLICE,
-
-    /**
-     * Get the object at the current position.  If the input is an
-     * improper list, the terminator is not returned in reply to this
-     * message.  Use TRL_GET_SUBLS() to retrieve the terminator in
-     * that case.
-     */
-    TR_MSG_GET_ELM,
-
-    /** Advance the iterator on the input. */
-    TR_MSG_NEXT,
-
-    /** Extract the product. */
-    TR_MSG_EXTRACT,
-
-    /** True iff the end of the sequence has been reached. */
-    TR_MSG_ENDP,
-
-    /**
-     * Splice OBJ and discard all cells at or after the current one
-     * in the input.  Only implemented for list translators.
-     */
-    TRL_MSG_SET_SUBLS,
-
-    TR_MSG_USR
-#define TR_BOOL_MSG_P(m) ((m) == TR_MSG_ENDP)
-};
-
-typedef enum _tr_msg tr_msg;
-typedef struct _tr_param tr_param;
-typedef struct _list_translator list_translator;
-typedef struct _vector_translator vector_translator;
-typedef struct _sequence_translator sequence_translator;
-typedef union _translator_ret translator_ret;
-
-struct _tr_param {
-    tr_msg msg;
-    ScmObj obj;
-};
-
-struct _list_translator {
-    ScmObj output;
-    ScmObj cur;
-    ScmObj src;
-    ScmQueue q;
-};
-
-struct _vector_translator {
-    ScmObj src;
-    ScmObj diff;
-    ScmQueue q;                 /* Points to diff. */
-    int index;                  /* Current position. */
-    int growth;
-};
-
-struct _sequence_translator {
-    translator_ret (*trans)(sequence_translator *t, tr_msg msg, ScmObj obj);
-    union {
-        list_translator lst;
-        vector_translator vec;
-    } u;
-};
-
-union _translator_ret {
-    ScmObj object;
-    scm_bool boolean;
-};
-
-/*
- * Operations on translators.  If a list- or vector-specific macro has
- * the same name (sans prefix) as a polymorphic one, the former tends
- * to be faster.
- */
-
-/* List-specific macros. */
-#define TRL_INIT(_t, _in)     ((_t).u.lst.output = (_in),               \
-                               SCM_QUEUE_POINT_TO((_t).u.lst.q,         \
-                                                  (_t).u.lst.output),   \
-                               (_t).u.lst.src = (_in),                  \
-                               (_t).u.lst.cur = (_in),                  \
-                               (_t).trans = scm_listran)
-#define TRL_GET_ELM(_t)       (CAR((_t).u.lst.cur))
-#define TRL_NEXT(_t)          ((_t).u.lst.cur = CDR((_t).u.lst.cur))
-#define TRL_ENDP(_t)          (!CONSP((_t).u.lst.cur))
-#define TRL_GET_SUBLS(_t)     ((_t).u.lst.cur)
-#define TRL_SET_SUBLS(_t, _o) (TRL_CALL((_t), TRL_MSG_SET_SUBLS, (_o)))
-#define TRL_EXTRACT(_t)       ((_t).u.lst.output)
-#define TRL_CALL(_t, _m, _o)  (scm_listran(&(_t), (_m), (_o)))
-#define TRL_EXECUTE(_t, _p)   (SCM_ASSERT(!TR_BOOL_MSG_P((_p).msg)),          \
-                               scm_listran(&(_t), (_p).msg, (_p).obj).object)
-
-/* Vector-specific macros. */
-#define TRV_INIT(_t, _in)  ((_t).u.vec.diff = SCM_NULL,                 \
-                            SCM_QUEUE_POINT_TO((_t).u.vec.q,            \
-                                               (_t).u.vec.diff),        \
-                            (_t).u.vec.src = (_in),                     \
-                            (_t).u.vec.index = 0,                       \
-                            (_t).u.vec.growth = 0,                      \
-                            (_t).trans = scm_vectran)
-#define TRV_GET_ELM(_t)    (SCM_VECTOR_VEC((_t).u.vec.src)[(_t).u.vec.index])
-#define TRV_NEXT(_t)       (++(_t).u.vec.index)
-#define TRV_GET_INDEX(_t)  ((_t).u.vec.index)
-#define TRV_GET_VEC(_t)    (SCM_VECTOR_VEC((_t).u.vec.src))
-#define TRV_ENDP(_t)       (SCM_VECTOR_LEN((_t).u.vec.src) <= (_t).u.vec.index)
-#define TRV_EXTRACT(_t)    (TRV_CALL((_t), TR_MSG_EXTRACT, SCM_INVALID).object)
-#define TRV_EXECUTE(_t, _p)  (TRV_CALL((_t), (_p).msg, (_p).obj).object)
-#define TRV_CALL(_t, _m, _o) (scm_vectran(&(_t), (_m), (_o)))
-
-/* Polymorphic macros. */
-#define TR_CALL(_t, _msg, _o) ((*(_t).trans)(&(_t), (_msg), (_o)))
-#define TR_EXECUTE(_t, _p) (TR_CALL((_t), (_p).msg, (_p).obj).object)
-#define TR_GET_ELM(_t)     (TR_CALL((_t), TR_MSG_GET_ELM, SCM_INVALID).object)
-#define TR_NEXT(_t)        ((void)TR_CALL((_t), TR_MSG_NEXT, SCM_INVALID))
-#define TR_ENDP(_t)        (TR_CALL((_t), TR_MSG_ENDP, SCM_INVALID).boolean)
-#define TR_EXTRACT(_t)     (TR_CALL((_t), TR_MSG_EXTRACT, SCM_INVALID).object)
-
 /*=======================================
   Local Buffer Allocator
 =======================================*/
-/* don't touch inside directly */
+/* don't touch member variables directly */
 #define ScmLBuf(T)                                                           \
     struct ScmLBuf_##T##_ {                                                  \
         T *buf;                                                              \
@@ -695,7 +561,7 @@ SCM_EXPORT void scm_lbuf_extend(struct ScmLBuf_void_ *lbuf,
 
 /*
  * extended size functions:
- * define your own version if more optimized version is needed
+ * define your own one if more optimized version is needed
  */
 SCM_EXPORT size_t scm_lbuf_f_linear(struct ScmLBuf_void_ *lbuf);
 SCM_EXPORT size_t scm_lbuf_f_exponential(struct ScmLBuf_void_ *lbuf);
@@ -716,8 +582,10 @@ struct ScmSpecialCharInfo_ {
 /* procedure.c */
 SCM_GLOBAL_VARS_BEGIN(procedure);
 ScmCharCodec *scm_identifier_codec;
+ScmObj scm_values_applier;
 SCM_GLOBAL_VARS_END(procedure);
 #define scm_identifier_codec SCM_GLOBAL_VAR(procedure, scm_identifier_codec)
+#define scm_values_applier   SCM_GLOBAL_VAR(procedure, scm_values_applier)
 SCM_DECLARE_EXPORTED_VARS(procedure);
 
 /* port.c */
@@ -775,8 +643,10 @@ SCM_EXPORT void scm_fin_storage(void);
 SCM_EXPORT void scm_init_gc(const ScmStorageConf *conf);
 SCM_EXPORT void scm_fin_gc(void);
 SCM_EXPORT ScmObj scm_alloc_cell(void);
+SCM_EXPORT void scm_prealloc_heaps(size_t n);
 
 /* continuation.c */
+#if SCM_USE_CONTINUATION
 SCM_EXPORT void scm_init_continuation(void);
 SCM_EXPORT void scm_fin_continuation(void);
 SCM_EXPORT void scm_destruct_continuation(ScmObj cont);
@@ -784,9 +654,14 @@ SCM_EXPORT ScmObj scm_call_with_current_continuation(ScmObj proc,
                                                      ScmEvalState *eval_state);
 SCM_EXPORT void scm_call_continuation(ScmObj cont, ScmObj ret) SCM_NORETURN;
 SCM_EXPORT ScmObj scm_dynamic_wind(ScmObj before, ScmObj thunk, ScmObj after);
+#if SCM_USE_BACKTRACE
 SCM_EXPORT void scm_push_trace_frame(ScmObj obj, ScmObj env);
 SCM_EXPORT void scm_pop_trace_frame(void);
+#endif /* SCM_USE_BACKTRACE */
 SCM_EXPORT ScmObj scm_trace_stack(void);
+#else /* SCM_USE_CONTINUATION */
+#define scm_trace_stack() SCM_NULL
+#endif /* SCM_USE_CONTINUATION */
 
 /* symbol.c */
 SCM_EXPORT void scm_init_symbol(const ScmStorageConf *conf);
@@ -820,51 +695,71 @@ SCM_EXPORT scm_bool scm_valid_environment_extension_lengthp(scm_int_t formals_le
 SCM_EXPORT scm_int_t scm_validate_formals(ScmObj formals);
 SCM_EXPORT scm_int_t scm_validate_actuals(ScmObj actuals);
 
-/* eval.c */
-SCM_EXPORT ScmObj scm_tailcall(ScmObj proc, ScmObj args,
-                               ScmEvalState *eval_state);
-
 /* syntax.c */
 SCM_EXPORT void scm_init_syntax(void);
+#if SCM_USE_INTERNAL_DEFINITIONS
 SCM_EXPORT ScmObj scm_s_body(ScmObj body, ScmEvalState *eval_state);
-SCM_EXPORT ScmObj scm_s_cond_internal(ScmObj args, ScmObj case_key,
+#else
+#define scm_s_body scm_s_begin
+#endif
+SCM_EXPORT ScmObj scm_s_cond_internal(ScmObj clauses,
                                       ScmEvalState *eval_state);
-SCM_EXPORT translator_ret scm_vectran(sequence_translator *t, tr_msg msg,
-                                      ScmObj obj);
-SCM_EXPORT translator_ret scm_listran(sequence_translator *t, tr_msg msg,
-                                      ScmObj obj);
+SCM_EXPORT ScmObj scm_s_let_internal(enum ScmObjType permitted,
+                                     ScmObj bindings, ScmObj body,
+                                     ScmEvalState *eval_state);
+SCM_EXPORT ScmObj scm_s_letrec_internal(enum ScmObjType permitted,
+                                        ScmObj bindings, ScmObj body,
+                                        ScmEvalState *eval_state);
+SCM_EXPORT void scm_s_define_internal(enum ScmObjType permitted,
+                                      ScmObj var, ScmObj exp, ScmObj env);
 
 /* macro.c */
+#if SCM_USE_HYGIENIC_MACRO
 SCM_EXPORT void scm_init_macro(void);
 SCM_EXPORT ScmObj scm_expand_macro(ScmObj macro, ScmObj args,
                                    ScmEvalState *eval_state);
 SCM_EXPORT ScmObj scm_p_reversex(ScmObj in); /* To be relocated. */
 SCM_EXPORT void scm_macro_bad_scope(ScmObj sym);
+#endif /* SCM_USE_HYGIENIC_MACRO */
 
 /* error.c */
 SCM_EXPORT void scm_init_error(void);
 
+/* promise.c */
+SCM_EXPORT void scm_init_promise(void);
+
+/* procedure.c */
+SCM_EXPORT ScmObj scm_map_single_arg(ScmObj proc, ScmObj lst);
+SCM_EXPORT ScmObj scm_map_multiple_args(ScmObj proc, ScmObj lsts,
+                                        scm_bool allow_uneven_lists);
+
 /* list.c */
 SCM_EXPORT scm_int_t scm_finite_length(ScmObj lst);
 
-/* number.c */
-SCM_EXPORT scm_int_t scm_string2number(const char *str, int radix,
-                                       scm_bool *err);
-
 /* port.c */
+#if SCM_USE_PORT
 SCM_EXPORT void scm_init_port(void);
 SCM_EXPORT ScmObj scm_prepare_port(ScmObj args, ScmObj default_port);
 SCM_EXPORT ScmCharPort *scm_make_char_port(ScmBytePort *bport);
+#endif /* SCM_USE_PORT */
 
 /* write.c */
+#if SCM_USE_WRITER
 SCM_EXPORT void scm_init_writer(void);
 SCM_EXPORT void scm_display_errobj_ss(ScmObj port, ScmObj errobj);
+#endif /* SCM_USE_WRITER */
 
 /* format.c */
+#if SCM_USE_FORMAT
 SCM_EXPORT void scm_init_format(void);
+#endif /* SCM_USE_FORMAT */
 
 /* load.c */
+#if SCM_USE_LOAD
 SCM_EXPORT void scm_init_load(void);
+SCM_EXPORT void scm_fin_load(void);
+SCM_EXPORT void scm_load_system_file(const char *file);
+#endif /* SCM_USE_LOAD */
 
 /* module.c */
 SCM_EXPORT void scm_init_module(void);
@@ -874,6 +769,91 @@ SCM_EXPORT void scm_fin_module(void);
 SCM_EXPORT char **scm_interpret_argv(char **argv);
 SCM_EXPORT void scm_free_argv(char **argv);
 
+/* legacy-macro.c */
+#if SCM_USE_LEGACY_MACRO
+SCM_EXPORT void scm_init_legacy_macro(void);
+#endif
+
+/*
+ * modules
+ */
+
+/* module-sscm-ext.c */
+#if SCM_USE_SSCM_EXTENSIONS
+SCM_EXPORT void scm_initialize_sscm_extensions(void);
+#endif
+
+/* module-siod.c */
+#if SCM_COMPAT_SIOD
+SCM_EXPORT void scm_initialize_siod(void);
+#endif
+
+/* module-srfi1.c */
+#if SCM_USE_SRFI1
+SCM_EXPORT void scm_initialize_srfi1(void);
+#endif
+
+/* module-srfi2.c */
+#if SCM_USE_SRFI2
+SCM_EXPORT void scm_initialize_srfi2(void);
+#endif
+
+/* module-srfi6.c */
+#if SCM_USE_SRFI6
+SCM_EXPORT void scm_initialize_srfi6(void);
+#endif
+
+/* module-srfi8.c */
+#if SCM_USE_SRFI8
+SCM_EXPORT void scm_initialize_srfi8(void);
+#endif
+
+/* module-srfi9.c */
+#if SCM_USE_SRFI9
+SCM_EXPORT void scm_initialize_srfi9(void);
+#endif
+
+/* module-srfi23.c */
+#if SCM_USE_SRFI23
+SCM_EXPORT void scm_initialize_srfi23(void);
+#endif
+
+/* module-srfi28.c */
+#if SCM_USE_SRFI28
+SCM_EXPORT void scm_initialize_srfi28(void);
+#endif
+
+/* module-srfi34.c */
+#if SCM_USE_SRFI34
+SCM_EXPORT void scm_initialize_srfi34(void);
+#endif
+
+/* module-srfi38.c */
+#if SCM_USE_SRFI38
+SCM_EXPORT void scm_initialize_srfi38(void);
+#endif
+
+/* module-srfi43.c */
+#if SCM_USE_SRFI43
+SCM_EXPORT void scm_initialize_srfi43(void);
+#endif
+
+/* module-srfi48.c */
+#if SCM_USE_SRFI48
+SCM_EXPORT void scm_initialize_srfi48(void);
+#endif
+
+/* module-srfi55.c */
+#if SCM_USE_SRFI55
+SCM_EXPORT void scm_initialize_srfi55(void);
+#endif
+
+/* module-srfi60.c */
+#if SCM_USE_SRFI60
+SCM_EXPORT void scm_initialize_srfi60(void);
+#endif
+
+
 #ifdef __cplusplus
 }
 #endif
