@@ -3,6 +3,9 @@
  *  About    : Garbage Collection
  *
  *  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
+ *  Copyright (C) 2005-2006 Jun Inoue <jun.lambda AT gmail.com>
+ *  Copyright (C) 2005-2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -39,7 +42,7 @@
  *   - gc_mark_locations()
  *       marks Scheme objects that held in the registers.
  *
- *   - gc_mark_locations()
+ *   - gc_mark_locations_n()
  *       marks Scheme objects that located on the stack.
  *
  *   - gc_mark_protected_var()
@@ -57,17 +60,23 @@
 
 #include <stddef.h>
 #include <stdlib.h>
-#include <setjmp.h>
 
-#include "sigscheme-stdint.h"
 #include "sigscheme.h"
 #include "sigschemeinternal.h"
 
 /*=======================================
   File Local Macro Definitions
 =======================================*/
-#define N_REGS_IN_JMP_BUF    (sizeof(jmp_buf) / sizeof(void *))
 #define SCMOBJ_ALIGNEDP(ptr) (!((uintptr_t)(ptr) % sizeof(ScmObj)))
+#if SCM_DEBUG
+#define SCM_BEGIN_GC_SUBCONTEXT() (scm_ensure_proper_freelist(l_freelist), \
+                                   ++l_gcing)
+#define SCM_END_GC_SUBCONTEXT()   (scm_ensure_proper_freelist(l_freelist), \
+                                   --l_gcing)
+#else /* not SCM_DEBUG */
+#define SCM_BEGIN_GC_SUBCONTEXT() SCM_EMPTY_EXPR
+#define SCM_END_GC_SUBCONTEXT()   SCM_EMPTY_EXPR
+#endif /* not SCM_DEBUG */
 
 /*=======================================
   File Local Type Definitions
@@ -77,8 +86,6 @@ typedef ScmCell *ScmObjHeap;
 /*=======================================
   Variable Definitions
 =======================================*/
-SCM_DEFINE_EXPORTED_VARS(gc);
-
 SCM_GLOBAL_VARS_BEGIN(static_gc);
 #define static
 static size_t l_heap_size, l_heap_alloc_threshold;
@@ -92,11 +99,13 @@ static ScmCell *l_heaps_lowest, *l_heaps_highest;
  *   -- YamaKen 2006-05-29 */
 static ScmObj l_freelist;
 
-static jmp_buf l_save_regs_buf;
-static ScmObj *l_stack_start_pointer;
-
 static ScmObj **l_protected_vars;
 static size_t l_protected_vars_size, l_n_empty_protected_vars;
+static GCROOTS_context *l_gcroots_ctx;
+#if SCM_DEBUG
+static size_t l_gcing;
+static scm_bool l_allocating;
+#endif /* SCM_DEBUG */
 #undef static
 SCM_GLOBAL_VARS_END(static_gc);
 #define l_heap_size            SCM_GLOBAL_VAR(static_gc, l_heap_size)
@@ -107,12 +116,15 @@ SCM_GLOBAL_VARS_END(static_gc);
 #define l_heaps_lowest         SCM_GLOBAL_VAR(static_gc, l_heaps_lowest)
 #define l_heaps_highest        SCM_GLOBAL_VAR(static_gc, l_heaps_highest)
 #define l_freelist             SCM_GLOBAL_VAR(static_gc, l_freelist)
-#define l_save_regs_buf        SCM_GLOBAL_VAR(static_gc, l_save_regs_buf)
-#define l_stack_start_pointer  SCM_GLOBAL_VAR(static_gc, l_stack_start_pointer)
 #define l_protected_vars       SCM_GLOBAL_VAR(static_gc, l_protected_vars)
 #define l_protected_vars_size  SCM_GLOBAL_VAR(static_gc, l_protected_vars_size)
 #define l_n_empty_protected_vars                                             \
     SCM_GLOBAL_VAR(static_gc, l_n_empty_protected_vars)
+#define l_gcroots_ctx          SCM_GLOBAL_VAR(static_gc, l_gcroots_ctx)
+#if SCM_DEBUG
+#define l_gcing                SCM_GLOBAL_VAR(static_gc, l_gcing)
+#define l_allocating           SCM_GLOBAL_VAR(static_gc, l_allocating)
+#endif /* SCM_DEBUG */
 SCM_DEFINE_STATIC_VARS(static_gc);
 
 /*=======================================
@@ -133,8 +145,10 @@ static scm_bool within_heapp(ScmObj obj);
 static void gc_mark_protected_var();
 static void gc_mark_locations_n(ScmObj *start, size_t n);
 static void gc_mark_definite_locations_n(ScmObj *start, size_t n);
-static void gc_mark_locations(ScmObj *start, ScmObj *end);
+static void gc_mark_locations(ScmObj *start, ScmObj *end,
+                              int is_certain, int is_aligned);
 static void gc_mark(void);
+static void gc_mark_global_vars(void);
 
 /* GC Sweep Related Functions */
 static void free_cell(ScmCell *cell);
@@ -142,17 +156,41 @@ static size_t gc_sweep(void);
 
 static void finalize_protected_var(void);
 
+#if SCM_DEBUG
+static void scm_ensure_proper_freelist(ScmObj flst);  /* FIXME */
+#endif
+
 /*=======================================
   Function Definitions
 =======================================*/
+#if SCM_DEBUG
+static void
+scm_ensure_proper_freelist(ScmObj flst)
+{
+    size_t len;
+    ScmObj c;
+
+    for (c = flst, len = 0; !SCM_NULLP(c); c = SCM_FREECELL_NEXT(c), len++) {
+        assert(SCM_FREECELLP(c));
+        assert(len <= SCM_INT_MAX);  /* not circular list */
+    }
+    SCM_ASSERT(SCM_NULLP(c));
+}
+#endif
+
 SCM_EXPORT void
 scm_init_gc(const ScmStorageConf *conf)
 {
-    SCM_GLOBAL_VARS_INIT(gc);
     SCM_GLOBAL_VARS_INIT(static_gc);
 
-    scm_gc_current_stack_fp = scm_gc_current_stack_internal;
-    scm_gc_protect_stack_fp = scm_gc_protect_stack_internal;
+#if SCM_DEBUG
+    l_gcing = 0;
+    l_allocating = scm_false;
+#endif
+
+    l_gcroots_ctx = GCROOTS_init(scm_malloc,
+                                 (GCROOTS_mark_proc)gc_mark_locations,
+                                 scm_false);
 
     initialize_heap(conf);
 }
@@ -163,24 +201,53 @@ scm_fin_gc(void)
     finalize_heap();
     finalize_protected_var();
 
-    SCM_GLOBAL_VARS_FIN(gc);
+    GCROOTS_fin(l_gcroots_ctx);
+    free(l_gcroots_ctx);
+
     SCM_GLOBAL_VARS_FIN(static_gc);
 }
 
 SCM_EXPORT ScmObj
 scm_alloc_cell(void)
 {
-    ScmObj ret = SCM_FALSE;
+    ScmObj ret;
+
+#if SCM_DEBUG
+    SCM_ASSERT(!l_gcing);
+    SCM_ASSERT(!l_allocating);
+    l_allocating = scm_true;
+#endif
 
     if (NULLP(l_freelist))
         gc_mark_and_sweep();
+    SCM_ASSERT(SCM_FREECELLP(l_freelist));
 
     ret = l_freelist;
     l_freelist = SCM_FREECELL_NEXT(l_freelist);
 
+#if SCM_DEBUG
+    l_allocating = scm_false;
+#endif
+
     return ret;
 }
 
+SCM_EXPORT void
+scm_prealloc_heaps(size_t n)
+{
+    size_t i;
+
+    if (!n)
+        n = l_n_heaps + 1;
+
+    if (n > l_n_heaps_max)
+        PLAIN_ERR("heap number ~ZU exceeded maxmum number ~ZU",
+                  n, l_n_heaps_max);
+
+    for (i = l_n_heaps; i < n; i++)
+        add_heap();
+}
+
 /*===========================================================================
   ScmObj Protection
 ===========================================================================*/
@@ -243,45 +310,76 @@ scm_gc_unprotect(ScmObj *var)
     }
 }
 
-/*===========================================================================
-  C Stack Protection
-===========================================================================*/
-/* scm_gc_current_stack_internal() is separated from
- * scm_gc_protect_stack_internal() to avoid returning inaccurate stack-start
- * address. Don't add any code fragments such as SCM_ASSERT() to this
- * function. It may alter the stack address.  -- YamaKen 2006-06-04 */
-SCM_EXPORT ScmObj *
-scm_gc_current_stack_internal(void)
+/* Usage:
+ *
+ *   assert(scm_gc_protectedp(obj));
+ *
+ * Don't use this predicate to ensure that an object is UNPROTECTED:
+ *
+ *   assert(!scm_gc_protectedp(obj));  // may cause unexpected failure
+ *
+ * This predicate cannot ensure that an object is UNPROTECTED, when
+ * scm_gc_protected_contextp() is true. i.e. An object may be identified as
+ * protected even if it is not placed on directly or indirectly protected
+ * location since an expired register variable may refer the object.
+ *   -- YamaKen 2007-03-25 */
+/* Though immediate values and symbols are GC safe even if not being
+ * explicitly protected, the condition may vary according to build
+ * configuration or future specification changes. So libsscm users should
+ * explicitly protect such objects.  -- YamaKen 2007-01-26 */
+SCM_EXPORT scm_bool
+scm_gc_protectedp(ScmObj obj)
 {
-    /*
-     * &stack_start will be relocated to start of the frame of subsequent
-     * function call
-     */
-    ScmObj stack_start;
+    ScmObj **slot;
 
-    /* intentionally returns invalidated local address with a warning
-     * suppression workaround */
-    return (ScmObj *)(((uintptr_t)&stack_start | 1) ^ 1);
+    /* constants or objects referred from registers or stack */
+    if (
+#if SCM_USE_STORAGE_COMPACT
+        SCM_IMMP(obj)
+#else
+        SCM_CONSTANTP(obj)
+#endif
+        || GCROOTS_is_protected(l_gcroots_ctx, (void *)obj))
+        return scm_true;
+
+    /* referred from static variables */
+    if (l_protected_vars) {
+        for (slot = l_protected_vars;
+             slot < &l_protected_vars[l_protected_vars_size];
+             slot++)
+        {
+            if (*slot && **slot == obj)
+                return scm_true;
+        }
+    }
+
+    /* referred from on-heap objects */
+    if (scm_gc_protected_contextp()) {
+        /* mark registers, stack and global vars */
+        gc_mark();
+    } else {
+        /* doesn't mark registers and stack */
+        gc_mark_global_vars();
+    }
+    gc_sweep();
+
+#if SCM_USE_STORAGE_COMPACT
+    return !SCM_CELL_FREECELLP(SCM_UNTAG_PTR(obj));
+#else
+    return !SCM_FREECELLP(obj);
+#endif
 }
 
-SCM_EXPORT ScmObj *
-scm_gc_protect_stack_internal(ScmObj *designated_stack_start)
+SCM_EXPORT void *
+scm_call_with_gc_ready_stack(ScmGCGateFunc func, void *arg)
 {
-    SCM_ASSERT(designated_stack_start);
-
-    if (!l_stack_start_pointer)
-        l_stack_start_pointer = designated_stack_start;
-
-    SCM_ASSERT(SCMOBJ_ALIGNEDP(l_stack_start_pointer));
-
-    return l_stack_start_pointer;
+    return GCROOTS_call_with_gc_ready_stack(l_gcroots_ctx, func, arg);
 }
 
-SCM_EXPORT void
-scm_gc_unprotect_stack(ScmObj *stack_start)
+SCM_EXPORT scm_bool
+scm_gc_protected_contextp(void)
 {
-    if (l_stack_start_pointer == stack_start)
-        l_stack_start_pointer = NULL;
+  return GCROOTS_is_protected_context(l_gcroots_ctx);
 }
 
 /*===========================================================================
@@ -290,19 +388,22 @@ scm_gc_unprotect_stack(ScmObj *stack_start)
 static void
 initialize_heap(const ScmStorageConf *conf)
 {
-    size_t i;
-
     l_heap_size            = conf->heap_size;
     l_heap_alloc_threshold = conf->heap_alloc_threshold;
     l_n_heaps_max          = conf->n_heaps_max;
     l_n_heaps = 0;
     l_heaps = NULL;
-    l_heaps_lowest = l_heaps_highest = NULL;
+    l_heaps_lowest = (void *)UINTPTR_MAX;
+    l_heaps_highest = NULL;
     l_freelist = SCM_NULL;
 
-    /* preallocate heaps */
-    for (i = 0; i < conf->n_heaps_init; i++)
-        add_heap();
+    /* Since maximum length of list can be represented by a Scheme integer,
+     * SCM_INT_MAX limits the number of cons cells. */
+    if (SCM_INT_MAX < l_n_heaps_max * l_heap_size
+        || SCM_INT_MAX < conf->n_heaps_init * l_heap_size)
+        scm_fatal_error("too large heap size specified");
+
+    scm_prealloc_heaps(conf->n_heaps_init);
 }
 
 static void
@@ -310,6 +411,9 @@ add_heap(void)
 {
     ScmObjHeap heap;
     ScmCell *cell;
+    ScmObj next;
+
+    SCM_BEGIN_GC_SUBCONTEXT();
 
     if (l_n_heaps_max <= l_n_heaps)
         scm_fatal_error("heap exhausted");
@@ -324,11 +428,13 @@ add_heap(void)
     if (&heap[0] < l_heaps_lowest)
         l_heaps_lowest = &heap[0];
 
-    /* link in order */
-    for (cell = &heap[0]; cell < &heap[l_heap_size - 1]; cell++)
-        SCM_RECLAIM_CELL(cell, cell + 1);
-    SCM_RECLAIM_CELL(cell, l_freelist);
-    l_freelist = (ScmObj)heap;
+    /* link as address-increasing order */
+    next = l_freelist;
+    for (cell = &heap[l_heap_size - 1]; cell >= &heap[0]; cell--)
+        next = SCM_CELL_RECLAIM_CELL(cell, next);
+    l_freelist = next;
+
+    SCM_END_GC_SUBCONTEXT();
 }
 
 static void
@@ -352,15 +458,19 @@ gc_mark_and_sweep(void)
 {
     size_t n_collected;
 
+    SCM_BEGIN_GC_SUBCONTEXT();
+
     CDBG((SCM_DBG_GC, "[ gc start ]"));
 
     gc_mark();
     n_collected = gc_sweep();
 
     if (n_collected < l_heap_alloc_threshold) {
-        CDBG((SCM_DBG_GC, "Cannot sweep the object, allocating new heap."));
+        CDBG((SCM_DBG_GC, "enough number of free cells cannot be collected. allocating new heap."));
         add_heap();
     }
+
+    SCM_END_GC_SUBCONTEXT();
 }
 
 
@@ -368,8 +478,10 @@ gc_mark_and_sweep(void)
 static void
 mark_obj(ScmObj obj)
 {
+#if SCM_USE_VECTOR
     scm_int_t i, len;
     ScmObj *vec;
+#endif
 
 mark_loop:
     /* no need to mark immediates */
@@ -407,17 +519,18 @@ mark_loop:
             obj = SCM_WRAPPER_OBJ(obj);
             goto mark_loop;
 #endif /* SCM_USE_HYGIENIC_MACRO */
-
+#if SCM_USE_VECTOR
         /* Alert: objects that store a non-ScmObj in obj_x must
          * explicitly drop the GC bit here.  This currently applies
          * only to vectors. */
         } else if (VECTORP(obj)) {
             len = SCM_VECTOR_LEN(obj);
             vec = SCM_VECTOR_VEC(obj);
-            vec = (ScmObj*)SCM_DROP_GCBIT((scm_intobj_t)vec);
+            vec = (ScmObj *)SCM_DROP_GCBIT((scm_intobj_t)vec);
             for (i = 0; i < len; i++) {
                 mark_obj(vec[i]);
             }
+#endif /* SCM_USE_VECTOR */
         } else if (VALUEPACKETP(obj)) {
             obj = SCM_VALUEPACKET_VALUES(obj);
             goto mark_loop;
@@ -428,11 +541,13 @@ mark_loop:
         break;
     }
 }
-#else /* SCM_USE_STORAGE_COMPACT */
+#elif SCM_USE_STORAGE_FATTY
 static void
 mark_obj(ScmObj obj)
 {
+#if SCM_USE_VECTOR
     scm_int_t i;
+#endif
 
 mark_loop:
     /* no need to mark constants */
@@ -487,17 +602,21 @@ mark_loop:
 #endif
         goto mark_loop;
 
+#if SCM_USE_VECTOR
     case ScmVector:
         for (i = 0; i < SCM_VECTOR_LEN(obj); i++) {
             mark_obj(SCM_VECTOR_VEC(obj)[i]);
         }
         break;
+#endif
 
     default:
         break;
     }
 }
-#endif /* SCM_USE_STORAGE_COMPACT */
+#else
+#error "mark_obj() is not implemented for this storage"
+#endif
 
 static void
 finalize_protected_var(void)
@@ -525,7 +644,7 @@ within_heapp(ScmObj obj)
     /*
      * Reject by rough conditions:
      * - heaps must be aligned to sizeof(ScmCell)
-     * - ptr is pointing to outside the enclosure which contain all heaps
+     * - ptr is pointing to outside of the enclosure which covers all heaps
      */
     if (((uintptr_t)ptr % sizeof(ScmCell))
         || (ptr < l_heaps_lowest || l_heaps_highest <= ptr))
@@ -551,6 +670,8 @@ gc_mark_protected_var(void)
 {
     ScmObj **slot;
 
+    SCM_BEGIN_GC_SUBCONTEXT();
+
     if (l_protected_vars) {
         for (slot = l_protected_vars;
              slot < &l_protected_vars[l_protected_vars_size];
@@ -560,6 +681,8 @@ gc_mark_protected_var(void)
                 mark_obj(**slot);
         }
     }
+
+    SCM_END_GC_SUBCONTEXT();
 }
 
 /* mark a contiguous region such as stack */
@@ -568,12 +691,16 @@ gc_mark_locations_n(ScmObj *start, size_t n)
 {
     ScmObj *objp;
 
+    SCM_BEGIN_GC_SUBCONTEXT();
+
     SCM_ASSERT(SCMOBJ_ALIGNEDP(start));
 
     for (objp = start; objp < &start[n]; objp++) {
         if (within_heapp(*objp))
             mark_obj(*objp);
     }
+
+    SCM_END_GC_SUBCONTEXT();
 }
 
 static void
@@ -581,51 +708,84 @@ gc_mark_definite_locations_n(ScmObj *start, size_t n)
 {
     ScmObj *objp;
 
+    SCM_BEGIN_GC_SUBCONTEXT();
+
     SCM_ASSERT(SCMOBJ_ALIGNEDP(start));
 
     for (objp = start; objp < &start[n]; objp++)
         mark_obj(*objp);
+
+    SCM_END_GC_SUBCONTEXT();
 }
 
 static void
-gc_mark_locations(ScmObj *start, ScmObj *end)
+gc_mark_locations(ScmObj *start, ScmObj *end, int is_certain, int is_aligned)
 {
-    ptrdiff_t size;
-    ScmObj *tmp;
+    ScmObj *adjusted_start, *tmp;
+    ptrdiff_t len;
+    unsigned int offset;
 
-    SCM_ASSERT(SCMOBJ_ALIGNEDP(end));
+    SCM_BEGIN_GC_SUBCONTEXT();
 
     /* swap end and start if (end < start) */
     if (end < start) {
-        tmp = end;
-        end = start;
+        tmp = end - 1;
+        end = start + 1;
         start = tmp;
     }
 
-    size = end - start;
+    /*
+     * workaround for non-aligned ScmObj on stack:
+     *
+     * Some architectures such as m68k does not align machine-word-size data on
+     * stack with its own size (i.e. 2byte-aligned 4byte-length words are
+     * used). So we scans the stack region (sizeof(ScmObj) / alignof(ScmObj))
+     * times with possible offsets. Since this implementation adopted the
+     * conservative GC strategy, it will not be a problem.
+     *   -- YamaKen 2006-12-09
+     */
+    for (offset = 0; offset < sizeof(ScmObj); offset += ALIGNOF_SCMOBJ) {
+        adjusted_start = (ScmObj *)((char *)start + offset);
+        len = end - adjusted_start;
+        CDBG((SCM_DBG_GC, "gc_mark_locations: start = ~P, end = ~P, len = ~TD, offset = ~U",
+              adjusted_start, end, len, offset));
 
-    CDBG((SCM_DBG_GC, "gc_mark_locations: size = ~TD", size));
+        if (is_certain)
+            gc_mark_definite_locations_n(adjusted_start, len);
+        else
+            gc_mark_locations_n(adjusted_start, len);
 
-    gc_mark_locations_n(start, size);
+        if (is_aligned)
+            break;
+    }
+
+    SCM_END_GC_SUBCONTEXT();
 }
 
-/*
- * To avoid incorrect stack_end placement, the jmp_buf is allocated to outside
- * of stack
- */
 static void
 gc_mark(void)
 {
-    ScmObj stack_end;
+    SCM_BEGIN_GC_SUBCONTEXT();
 
-    setjmp(l_save_regs_buf);
-    gc_mark_locations_n((ScmObj *)l_save_regs_buf, N_REGS_IN_JMP_BUF);
-    gc_mark_locations(l_stack_start_pointer, &stack_end);
+    /* Mark stack and all machine-dependent contexts such as registers,
+     * register windows (SPARC), register stack backing store (IA-64) etc. */
+    GCROOTS_mark(l_gcroots_ctx);
+
+    gc_mark_global_vars();
+
+    SCM_END_GC_SUBCONTEXT();
+}
+
+static void
+gc_mark_global_vars(void)
+{
+    SCM_BEGIN_GC_SUBCONTEXT();
 
-    /* performed after above two because of cache pollution */
     gc_mark_protected_var();
     if (scm_symbol_hash)
         gc_mark_definite_locations_n(scm_symbol_hash, scm_symbol_hash_size);
+
+    SCM_END_GC_SUBCONTEXT();
 }
 
 static void
@@ -635,44 +795,63 @@ free_cell(ScmCell *cell)
     if (SCM_CELL_MISCP(*cell)) {
         if (SCM_CELL_SYMBOLP(*cell))
             SCM_CELL_SYMBOL_FIN(*cell);
+#if SCM_USE_STRING
         else if (SCM_CELL_STRINGP(*cell))
             SCM_CELL_STRING_FIN(*cell);
+#endif
+#if SCM_USE_VECTOR
         else if (SCM_CELL_VECTORP(*cell))
             SCM_CELL_VECTOR_FIN(*cell);
+#endif
+#if SCM_USE_PORT
         else if (SCM_CELL_PORTP(*cell))
             SCM_CELL_PORT_FIN(*cell);
+#endif
+#if SCM_USE_CONTINUATION
         else if (SCM_CELL_CONTINUATIONP(*cell))
             SCM_CELL_CONTINUATION_FIN(*cell);
+#endif
     }
 #else /* SCM_USE_STORAGE_COMPACT */
     switch (SCM_TYPE(cell)) {
     case ScmCons:
+#if SCM_USE_INT
     case ScmInt:
+#endif
+#if SCM_USE_CHAR
     case ScmChar:
-    case ScmClosure:
-        break;
-
-    case ScmString:
-        if (SCM_STRING_STR(cell))
-            free(SCM_STRING_STR(cell));
-        break;
-
-    case ScmVector:
-        if (SCM_VECTOR_VEC(cell))
-            free(SCM_VECTOR_VEC(cell));
+#endif
         break;
 
     case ScmSymbol:
-        if (SCM_SYMBOL_NAME(cell))
-            free(SCM_SYMBOL_NAME(cell));
+        free(SCM_SYMBOL_NAME(cell));
         break;
 
+#if SCM_USE_STRING
+    case ScmString:
+        free(SCM_STRING_STR(cell));
+        break;
+#endif
+
+    case ScmFreeCell:
+    case ScmConstant:
+        break;
+
+#if SCM_USE_VECTOR
+    case ScmVector:
+        free(SCM_VECTOR_VEC(cell));
+        break;
+#endif
+
+    /* rarely swept objects */
+#if SCM_USE_PORT
     case ScmPort:
         if (SCM_PORT_IMPL(cell))
             scm_port_close(cell);
         break;
+#endif
 
-    /* rarely swept objects */
+#if SCM_USE_CONTINUATION
     case ScmContinuation:
         /*
          * Since continuation object is not so many, destructing the object by
@@ -683,12 +862,24 @@ free_cell(ScmCell *cell)
          */
         scm_destruct_continuation(cell);
         break;
+#endif
 
+    case ScmClosure:
     case ScmFunc:
-    case ScmConstant:
-    case ScmFreeCell:
-    default:
+    case ScmMacro:
+    case ScmFarsymbol:
+    case ScmSubpat:
+    case ScmCFuncPointer:
+    case ScmCPointer:
+    case ScmValuePacket:
         break;
+#if SCM_DEBUG
+    case ScmRational:
+    case ScmReal:
+    case ScmComplex:
+#endif
+    default:
+        SCM_NOTREACHED;
     }
 #endif /* SCM_USE_STORAGE_COMPACT */
 }
@@ -699,7 +890,9 @@ gc_sweep(void)
     size_t i, sum_collected, n_collected;
     ScmObjHeap heap;
     ScmCell *cell;
-    ScmObj obj, new_freelist;
+    ScmObj new_freelist;
+
+    SCM_BEGIN_GC_SUBCONTEXT();
 
     /* Because l_freelist may not be exhausted on an user-instructed GC, do not
      * assume that l_freelist is null here. -- YamaKen */
@@ -711,18 +904,13 @@ gc_sweep(void)
         heap = l_heaps[i];
 
         for (cell = &heap[0]; cell < &heap[l_heap_size]; cell++) {
-            /* FIXME: is this safe for SCM_USE_STORAGE_COMPACT? */
-            /* Yes, but it's probably cleaner to change SCM_MARKEDP()
-             * et al to SCM_CELL_MARKEDP() etc and take care of
-             * dereferencing in this file.  -- Jun Inoue */
-            obj = (ScmObj)cell;
-
-            if (SCM_MARKEDP(obj)) {
-                SCM_UNMARK(obj);
-            } else {
+            if (SCM_CELL_MARKEDP(cell)) {
+                SCM_CELL_UNMARK(cell);
+            } else if (!SCM_CELL_FREECELLP(cell)) {
+                /* scm_gc_protectedp() causes GC sweep on heaps that contain
+                 * freecells. So !SCM_CELL_FREECELLP(cell) is required. */
                 free_cell(cell);
-                SCM_RECLAIM_CELL(cell, new_freelist);
-                new_freelist = (ScmObj)cell;
+                new_freelist = SCM_CELL_RECLAIM_CELL(cell, new_freelist);
                 n_collected++;
             }
         }
@@ -732,5 +920,7 @@ gc_sweep(void)
     }
     l_freelist = new_freelist;
 
+    SCM_END_GC_SUBCONTEXT();
+
     return sum_collected;
 }
