@@ -3,6 +3,9 @@
  *  About    : scheme storage layer
  *
  *  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
+ *  Copyright (C) 2005-2006 Jun Inoue <jun.lambda AT gmail.com>
+ *  Copyright (C) 2005-2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
+ *  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
  *
  *  All rights reserved.
  *
@@ -34,14 +37,15 @@
 
 #include <config.h>
 
-#if !SCM_USE_MULTIBYTE_CHAR
-#include <string.h>
-#endif
 #include <stdlib.h>
 
 #include "sigscheme.h"
 #include "sigschemeinternal.h"
+#if SCM_USE_MULTIBYTE_CHAR
 #include "encoding.h"
+#else
+#include "encoding-dummy.h"
+#endif
 
 /*=======================================
   File Local Macro Definitions
@@ -55,9 +59,9 @@
     } while (/* CONSTCOND */ 0)
 #endif /* SCM_USE_STORAGE_FATTY */
 
-/* FIXME: SCM_*_INIT() macros should not be a SAL interface because it taints
- * the SAL abstraction model with an unwanted restriction. It should be an
- * internal interface of storage-compact and storage-fatty.
+/* SCM_*_INIT() macros should not be a SAL interface because it taints the SAL
+ * abstraction model with an unwanted restriction. It should be an internal
+ * interface of storage-compact and storage-fatty.
  *
  * Since the INIT() interface assumes that any cell of a storage is generic and
  * non-colored, and requires 'all-purpose' cell allocation ability. The
@@ -69,114 +73,119 @@
  * maximum storage optimization capability based on platform-by-platform system
  * characteristics (especially on embedded platforms), the restriction is
  * difficult to accept.  -- YamaKen 2006-05-30  */
+/* SCM_SAL_*_INIT()s are renamed to SCM_ISAL_*_INIT() to distinguish them from
+ * SAL. ISAL stands for "Internal SAL".  -- YamaKen 2006-06-24 */
 #define SCM_CONS_INIT(obj, kar, kdr)                    \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_CONS_INIT,          \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_CONS_INIT,         \
                             (ScmObj, ScmObj, ScmObj),   \
                             ((obj), (kar), (kdr)))
 
 #define SCM_IMMUTABLE_CONS_INIT(obj, kar, kdr)                  \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_IMMUTABLE_CONS_INIT,        \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_IMMUTABLE_CONS_INIT,       \
                             (ScmObj, ScmObj, ScmObj),           \
                             ((obj), (kar), (kdr)))
 
 #define SCM_CLOSURE_INIT(obj, exp, env)                 \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_CLOSURE_INIT,       \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_CLOSURE_INIT,      \
                             (ScmObj, ScmObj, ScmObj),   \
                             ((obj), (exp), (env)))
 
 #if !SCM_HAS_IMMEDIATE_CHAR_ONLY
-#define SCM_CHAR_INIT(o, val)                           \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_CHAR_INIT,          \
+#define SCM_CHAR_INIT(obj, val)                         \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_CHAR_INIT,         \
                             (ScmObj, scm_ichar_t),      \
-                            ((o), (val)))
+                            ((obj), (val)))
 #endif
 
 #if !SCM_HAS_IMMEDIATE_INT_ONLY
-#define SCM_INT_INIT(o, val)                            \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_INT_INIT,           \
+#define SCM_INT_INIT(obj, val)                          \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_INT_INIT,          \
                             (ScmObj, scm_int_t),        \
-                            ((o), (val)))
+                            ((obj), (val)))
 #endif
 
-#define SCM_SYMBOL_INIT(obj, nam, val)                  \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_SYMBOL_INIT,        \
-                            (ScmObj, char*, ScmObj),    \
-                            ((obj), (nam), (val)))
+#define SCM_SYMBOL_INIT(obj, name, val)                 \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_SYMBOL_INIT,       \
+                            (ScmObj, char *, ScmObj),   \
+                            ((obj), (name), (val)))
 
 #define SCM_STRING_INIT(obj, str, len, mutp)                            \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_STRING_INIT,                        \
-                            (ScmObj, char*, scm_int_t, scm_bool),       \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_STRING_INIT,                       \
+                            (ScmObj, char *, scm_int_t, scm_bool),      \
                             ((obj), (str), (len), (mutp)))
 
 #define SCM_MUTABLE_STRING_INIT(obj, str, len)                  \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_MUTABLE_STRING_INIT,        \
-                            (ScmObj, char*, scm_int_t),         \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_MUTABLE_STRING_INIT,       \
+                            (ScmObj, char *, scm_int_t),        \
                             ((obj), (str), (len)))
 
 #define SCM_IMMUTABLE_STRING_INIT(obj, str, len)                \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_IMMUTABLE_STRING_INIT,      \
-                            (ScmObj, char*, scm_int_t),         \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_IMMUTABLE_STRING_INIT,     \
+                            (ScmObj, char *, scm_int_t),        \
                             ((obj), (str), (len)))
 
 #define SCM_FUNC_INIT(obj, type, func)                                   \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_FUNC_INIT,                           \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_FUNC_INIT,                          \
                             (ScmObj, enum ScmFuncTypeCode, ScmFuncType), \
                             ((obj), (type), (func)))
+
 #define SCM_VECTOR_INIT(obj, vec, len)                          \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_VECTOR_INIT,                \
-                            (ScmObj, ScmObj*, scm_int_t),       \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_VECTOR_INIT,               \
+                            (ScmObj, ScmObj *, scm_int_t),      \
                             ((obj), (vec), (len)))
 
 #define SCM_MUTABLE_VECTOR_INIT(obj, vec, len)                  \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_MUTABLE_VECTOR_INIT,        \
-                            (ScmObj, ScmObj*, scm_int_t),       \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_MUTABLE_VECTOR_INIT,       \
+                            (ScmObj, ScmObj *, scm_int_t),      \
                             ((obj), (vec), (len)))
 
 #define SCM_IMMUTABLE_VECTOR_INIT(obj, vec, len)                \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_IMMUTABLE_VECTOR_INIT,      \
-                            (ScmObj, ScmObj*, scm_int_t),       \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_IMMUTABLE_VECTOR_INIT,     \
+                            (ScmObj, ScmObj *, scm_int_t),      \
                             ((obj), (vec), (len)))
+
 #define SCM_PORT_INIT(obj, cport, flag)                         \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_PORT_INIT,                  \
-                            (ScmObj, struct ScmCharPort_*,      \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_PORT_INIT,                 \
+                            (ScmObj, struct ScmCharPort_ *,     \
                              enum ScmPortFlag),                 \
                             ((obj), (cport), (flag)))
-#define SCM_CONTINUATION_INIT(obj, oq, tag)             \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_CONTINUATION_INIT,  \
-                            (ScmObj, void*, scm_int_t), \
-                            ((obj), (oq), (tag)))
+
+#define SCM_CONTINUATION_INIT(obj, opaque, tag)          \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_CONTINUATION_INIT,  \
+                            (ScmObj, void *, scm_int_t), \
+                            ((obj), (opaque), (tag)))
 
 #if SCM_USE_SSCM_EXTENSIONS
 /* SCM_C_POINTER_INIT(obj, void *ptr) */
 #define SCM_C_POINTER_INIT(obj, ptr)                    \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_C_POINTER_INIT,     \
-                            (ScmObj, void*),            \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_C_POINTER_INIT,    \
+                            (ScmObj, void *),           \
                             ((obj), (ptr)))
 /* SCM_C_FUNCPOINTER_INIT(obj, ScmCFunc ptr) */
-#define SCM_C_FUNCPOINTER_INIT(obj, ptr)                \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_C_FUNCPOINTER_INIT, \
-                            (ScmObj, ScmCFunc),         \
+#define SCM_C_FUNCPOINTER_INIT(obj, ptr)                 \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_C_FUNCPOINTER_INIT, \
+                            (ScmObj, ScmCFunc),          \
                             ((obj), (ptr)))
 #endif /* SCM_USE_SSCM_EXTENSIONS */
 #define SCM_VALUEPACKET_INIT(obj, vals)                 \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_VALUEPACKET_INIT,   \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_VALUEPACKET_INIT,  \
                             (ScmObj, ScmObj),           \
                             ((obj), (vals)))
 
-#define SCM_HMACRO_INIT(o, r, e)                                \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_HMACRO_INIT,                \
+#define SCM_HMACRO_INIT(obj, r, e)                              \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_HMACRO_INIT,               \
                             (ScmObj, ScmObj, ScmPackedEnv),     \
-                            ((o), (r), (e)))
+                            ((obj), (r), (e)))
 
-#define SCM_FARSYMBOL_INIT(o, s, e)                             \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_FARSYMBOL_INIT,             \
+#define SCM_FARSYMBOL_INIT(obj, s, e)                           \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_FARSYMBOL_INIT,            \
                             (ScmObj, ScmObj, ScmPackedEnv),     \
-                            ((o), (s), (e)))
+                            ((obj), (s), (e)))
 
-#define SCM_SUBPAT_INIT(o, x, m)                                \
-    SCM_TYPESAFE_MACRO_VOID(SCM_SAL_SUBPAT_INIT,                \
+#define SCM_SUBPAT_INIT(obj, x, m)                              \
+    SCM_TYPESAFE_MACRO_VOID(SCM_ISAL_SUBPAT_INIT,               \
                             (ScmObj, ScmObj, scm_int_t),        \
-                            ((o), (x), (m)))
+                            ((obj), (x), (m)))
 
 /*=======================================
   File Local Type Definitions
@@ -221,9 +230,13 @@ static const ScmStorageConf default_storage_conf = {
 /*=======================================
   File Local Function Declarations
 =======================================*/
+#if SCM_USE_STORAGE_FATTY
 static void initialize_special_constants(void);
+#endif
+#if SCM_USE_STRING
 static ScmObj scm_make_string_internal(char *str, scm_int_t len,
-                                       scm_bool is_immutable);
+                                       scm_bool immutablep);
+#endif
 
 /*=======================================
   Function Definitions
@@ -237,29 +250,24 @@ scm_init_storage(const ScmStorageConf *conf)
 #if SCM_USE_STORAGE_FATTY
     SCM_GLOBAL_VARS_INIT(storage_fatty);
     SCM_GLOBAL_VARS_INIT(static_storage_fatty);
+
+    initialize_special_constants();
 #endif
 
     if (!conf)
         conf = &default_storage_conf;
-
-    initialize_special_constants();
-
     scm_init_gc(conf);
 
-#if 0 && (SCM_COMPAT_SIOD_BUGS && SCM_USE_STORAGE_FATTY)
-    scm_gc_protect_with_init(&scm_const_true, MAKE_INT(1));
-#endif
-
 #if SCM_USE_VALUECONS
-    /*
-     * To keep storage model abstract, the cell is allocated from a heap
-     * instead of directly construct ScmCell
-     */
+    /* To keep storage model abstract, the cell is allocated from a heap
+     * instead of directly construct ScmCell. */
     scm_gc_protect_with_init(&scm_null_values, CONS(SCM_NULL, SCM_NULL));
     SCM_ENTYPE(scm_null_values, ScmValuePacket);
 #endif
 
+#if SCM_USE_CONTINUATION
     scm_init_continuation();
+#endif
     scm_init_symbol(conf);
 }
 
@@ -267,14 +275,16 @@ SCM_EXPORT void
 scm_fin_storage(void)
 {
     scm_fin_symbol();
+#if SCM_USE_CONTINUATION
     scm_fin_continuation();
+#endif
     scm_fin_gc();
 
-    SCM_GLOBAL_VARS_FIN(storage);
 #if SCM_USE_STORAGE_FATTY
-    SCM_GLOBAL_VARS_FIN(storage_fatty);
     SCM_GLOBAL_VARS_FIN(static_storage_fatty);
+    SCM_GLOBAL_VARS_FIN(storage_fatty);
 #endif
+    SCM_GLOBAL_VARS_FIN(storage);
 }
 
 /*===========================================================================
@@ -285,18 +295,18 @@ scm_fin_storage(void)
  * initialization is encapsulated in this file. Upper layers must only use
  * abstract interfaces such as SCM_NULL and SCM_NULLP().
  */
+#if SCM_USE_STORAGE_FATTY
 static void
 initialize_special_constants(void)
 {
-#if SCM_USE_STORAGE_FATTY
     SCM_CONSTANT_BIND_SUBSTANCE(scm_const_null,    l_null_cell);
     SCM_CONSTANT_BIND_SUBSTANCE(scm_const_true,    l_true_cell);
     SCM_CONSTANT_BIND_SUBSTANCE(scm_const_false,   l_false_cell);
     SCM_CONSTANT_BIND_SUBSTANCE(scm_const_eof,     l_eof_cell);
     SCM_CONSTANT_BIND_SUBSTANCE(scm_const_unbound, l_unbound_cell);
     SCM_CONSTANT_BIND_SUBSTANCE(scm_const_undef,   l_undef_cell);
-#endif /* SCM_USE_STORAGE_FATTY */
 }
+#endif /* SCM_USE_STORAGE_FATTY */
 
 /*===========================================================================
   Object Allocators
@@ -311,7 +321,7 @@ scm_make_cons(ScmObj kar, ScmObj kdr)
     return obj;
 }
 
-#if SCM_SAL_HAS_IMMUTABLE_CONS
+#if SCM_HAS_IMMUTABLE_CONS
 SCM_EXPORT ScmObj
 scm_make_immutable_cons(ScmObj kar, ScmObj kdr)
 {
@@ -323,7 +333,7 @@ scm_make_immutable_cons(ScmObj kar, ScmObj kdr)
 }
 #endif /* has immutable cons */
 
-#if !SCM_SAL_HAS_IMMEDIATE_INT_ONLY
+#if (SCM_USE_INT && !SCM_HAS_IMMEDIATE_INT_ONLY)
 SCM_EXPORT ScmObj
 scm_make_int(scm_int_t val)
 {
@@ -333,7 +343,7 @@ scm_make_int(scm_int_t val)
     SCM_INT_INIT(obj, val);
     return obj;
 }
-#endif /* not SCM_SAL_HAS_IMMEDIATE_INT_ONLY */
+#endif /* (SCM_USE_INT && !SCM_HAS_IMMEDIATE_INT_ONLY) */
 
 SCM_EXPORT ScmObj
 scm_make_symbol(char *name, ScmObj val)
@@ -345,7 +355,7 @@ scm_make_symbol(char *name, ScmObj val)
     return obj;
 }
 
-#if !SCM_SAL_HAS_IMMEDIATE_CHAR_ONLY
+#if (SCM_USE_CHAR && !SCM_HAS_IMMEDIATE_CHAR_ONLY)
 SCM_EXPORT ScmObj
 scm_make_char(scm_ichar_t val)
 {
@@ -355,7 +365,7 @@ scm_make_char(scm_ichar_t val)
     SCM_CHAR_INIT(obj, val);
     return obj;
 }
-#endif /* not SCM_SAL_HAS_IMMEDIATE_INT_ONLY */
+#endif /* (SCM_USE_CHAR && !SCM_HAS_IMMEDIATE_CHAR_ONLY) */
 
 #if SCM_USE_HYGIENIC_MACRO
 SCM_EXPORT ScmObj
@@ -393,28 +403,24 @@ scm_make_subpat(ScmObj x, scm_int_t meta)
 }
 #endif /* SCM_USE_HYGIENIC_MACRO */
 
+#if SCM_USE_STRING
 static ScmObj
-scm_make_string_internal(char *str, scm_int_t len, scm_bool is_immutable)
+scm_make_string_internal(char *str, scm_int_t len, scm_bool immutablep)
 {
     ScmObj obj;
 
     SCM_ASSERT(str);
 
-    if (len == STRLEN_UNKNOWN) {
-#if SCM_USE_MULTIBYTE_CHAR
+    if (len == STRLEN_UNKNOWN)
         len = scm_mb_bare_c_strlen(scm_current_char_codec, str);
-#else
-        len = strlen(str);
-#endif
-    }
 
     obj = scm_alloc_cell();
-    SCM_STRING_INIT(obj, str, len, !is_immutable);
+    SCM_STRING_INIT(obj, str, len, !immutablep);
 
     return obj;
 }
 
-#if SCM_SAL_HAS_IMMUTABLE_STRING
+#if SCM_HAS_IMMUTABLE_STRING
 SCM_EXPORT ScmObj
 scm_make_immutable_string(char *str, scm_int_t len)
 {
@@ -426,7 +432,7 @@ scm_make_immutable_string_copying(const char *str, scm_int_t len)
 {
     return scm_make_string_internal(scm_strdup(str), len, scm_true);
 }
-#endif /* SCM_SAL_HAS_IMMUTABLE_STRING */
+#endif /* SCM_HAS_IMMUTABLE_STRING */
 
 SCM_EXPORT ScmObj
 scm_make_string(char *str, scm_int_t len)
@@ -439,6 +445,7 @@ scm_make_string_copying(const char *str, scm_int_t len)
 {
     return scm_make_string_internal(scm_strdup(str), len, scm_false);
 }
+#endif /* SCM_USE_STRING */
 
 SCM_EXPORT ScmObj
 scm_make_func(enum ScmFuncTypeCode type, ScmFuncType func)
@@ -460,6 +467,7 @@ scm_make_closure(ScmObj exp, ScmObj env)
     return obj;
 }
 
+#if SCM_USE_VECTOR
 SCM_EXPORT ScmObj
 scm_make_vector(ScmObj *vec, scm_int_t len)
 {
@@ -470,20 +478,20 @@ scm_make_vector(ScmObj *vec, scm_int_t len)
     return obj;
 }
 
-#if SCM_SAL_HAS_IMMUTABLE_VECTOR
+#if SCM_HAS_IMMUTABLE_VECTOR
 SCM_EXPORT ScmObj
 scm_make_immutable_vector(ScmObj *vec, scm_int_t len)
 {
     ScmObj obj;
 
-    /* Since this function is rarely used, the inefficiency is not a problem */
-    obj = scm_make_vector(vec, len);
-    SCM_VECTOR_SET_IMMUTABLE(obj);
-
+    obj = scm_alloc_cell();
+    SCM_IMMUTABLE_VECTOR_INIT(obj, vec, len);
     return obj;
 }
-#endif /* SCM_SAL_HAS_IMMUTABLE_VECTOR */
+#endif /* SCM_HAS_IMMUTABLE_VECTOR */
+#endif /* SCM_USE_VECTOR */
 
+#if SCM_USE_PORT
 SCM_EXPORT ScmObj
 scm_make_port(ScmCharPort *cport, enum ScmPortFlag flag)
 {
@@ -499,7 +507,9 @@ scm_make_port(ScmCharPort *cport, enum ScmPortFlag flag)
     SCM_PORT_INIT(obj, cport, flag);
     return obj;
 }
+#endif /* SCM_USE_PORT */
 
+#if (SCM_USE_CONTINUATION || SCM_USE_VALUES_APPLIER)
 SCM_EXPORT ScmObj
 scm_make_continuation(void)
 {
@@ -509,6 +519,7 @@ scm_make_continuation(void)
     SCM_CONTINUATION_INIT(obj, INVALID_CONTINUATION_OPAQUE, 0);
     return obj;
 }
+#endif /* (SCM_USE_CONTINUATION || SCM_USE_VALUES_APPLIER) */
 
 #if !SCM_USE_VALUECONS
 SCM_EXPORT ScmObj
@@ -558,34 +569,56 @@ scm_type(ScmObj obj)
     case SCM_PTAG_MISC:
         if (SYMBOLP(obj))
             return ScmSymbol;
+#if SCM_USE_STRING
         else if (STRINGP(obj))
             return ScmString;
+#endif
+#if SCM_USE_VECTOR
         else if (VECTORP(obj))
             return ScmVector;
+#endif
         else if (VALUEPACKETP(obj))
             return ScmValuePacket;
         else if (FUNCP(obj))
             return ScmFunc;
+#if SCM_USE_PORT
         else if (PORTP(obj))
             return ScmPort;
+#endif
+#if SCM_USE_CONTINUATION
         else if (CONTINUATIONP(obj))
             return ScmContinuation;
+#endif
         else if (SCM_CONSTANTP(obj))
             return ScmConstant;
+#if SCM_USE_SSCM_EXTENSIONS
         else if (C_POINTERP(obj))
             return ScmCPointer;
         else if (C_FUNCPOINTERP(obj))
             return ScmCFuncPointer;
+#endif
+#if SCM_USE_HYGIENIC_MACRO
+        else if (HMACROP(obj))
+            return ScmMacro;
+        else if (FARSYMBOLP(obj))
+            return ScmFarsymbol;
+        else if (SUBPATP(obj))
+            return ScmSubpat;
+#endif
         else if (FREECELLP(obj))
             return ScmFreeCell;
-        PLAIN_ERR(" invalid others object: ptr = ~P", (void *)obj);
+        PLAIN_ERR("invalid misc object: ptr = ~P", (void *)obj);
 
     case SCM_PTAG_IMM:
+#if SCM_USE_INT
         if (INTP(obj))
             return ScmInt;
-        else if (CHARP(obj))
+#endif
+#if SCM_USE_CHAR
+        if (CHARP(obj))
             return ScmChar;
-        else if (SCM_CONSTANTP(obj))
+#endif
+        if (SCM_CONSTANTP(obj))
             return ScmConstant;
         PLAIN_ERR("invalid imm object: ptr = ~P", (void *)obj);
 
