@@ -5,7 +5,7 @@
 ;;  About    : unit test for EUC-JP string
 ;;
 ;;  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
-;;  Copyright (c) 2007 SigScheme Project <uim AT freedesktop.org>
+;;  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
 ;;
 ;;  All rights reserved.
 ;;
@@ -34,7 +34,7 @@
 ;;  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-(load "./test/unittest.scm")
+(require-extension (unittest))
 
 (if (not (and (provided? "euc-jp")
               (symbol-bound? 'char?)
@@ -70,10 +70,9 @@
 						  (define str (string-copy "aieo"))
 						  (string-set! str 2 #\u)
 						  str)))
-(assert-equal? "string-set! 1"     "˶"   (string-set!
-                                               (string-copy "ˤ")
-                                               2
-                                               #\))
+(assert-equal? "string-set! 1"     "˶"   (let ((s (string-copy "ˤ")))
+                                                (string-set! s 2 #\)
+                                                s))
 
 
 ;; string-length
@@ -133,17 +132,121 @@
 (assert-equal? "string 2" str1 (apply string str1-list))
 (assert-equal? "list->string 2" str1-list (string->list str1))
 
-;; SRFI-75
+;; unsupported SRFI-75 literals cause error
 (tn "SRFI-75")
-(assert-parseable   (tn) "#\\x63")
+(assert-parseable (tn) "#\\x63")
 (assert-parse-error (tn) "#\\u0063")
 (assert-parse-error (tn) "#\\U00000063")
 
-(assert-parseable   (tn) "\"\\x63\"")
+(assert-parse-error (tn) "\"\\x63\"")
 (assert-parse-error (tn) "\"\\u0063\"")
 (assert-parse-error (tn) "\"\\U00000063\"")
 
 (assert-parseable   (tn) "'a")
+;; Non-Unicode multibyte symbols are not allowed.
 (assert-parse-error (tn) "'")
 
+(tn "R6RS (R5.92RS) chars")
+(assert-parseable   (tn) "#\\x")
+(assert-parseable   (tn) "#\\x6")
+(assert-parseable   (tn) "#\\xf")
+(assert-parseable   (tn) "#\\x63")
+(assert-parseable   (tn) "#\\x063")
+(assert-parseable   (tn) "#\\x0063")
+(assert-parseable   (tn) "#\\x00063")
+(assert-parseable   (tn) "#\\x0000063")
+(assert-parseable   (tn) "#\\x00000063")
+(assert-parse-error (tn) "#\\x000000063")
+
+(assert-parseable   (tn) "#\\x3042")
+
+(assert-parse-error (tn) "#\\x-")
+(assert-parse-error (tn) "#\\x-6")
+(assert-parse-error (tn) "#\\x-f")
+(assert-parse-error (tn) "#\\x-63")
+(assert-parse-error (tn) "#\\x-063")
+(assert-parse-error (tn) "#\\x-0063")
+(assert-parse-error (tn) "#\\x-00063")
+(assert-parse-error (tn) "#\\x-0000063")
+(assert-parse-error (tn) "#\\x-00000063")
+(assert-parse-error (tn) "#\\x-000000063")
+
+(assert-parse-error (tn) "#\\x+")
+(assert-parse-error (tn) "#\\x+6")
+(assert-parse-error (tn) "#\\x+f")
+(assert-parse-error (tn) "#\\x+63")
+(assert-parse-error (tn) "#\\x+063")
+(assert-parse-error (tn) "#\\x+0063")
+(assert-parse-error (tn) "#\\x+00063")
+(assert-parse-error (tn) "#\\x+0000063")
+(assert-parse-error (tn) "#\\x+00000063")
+(assert-parse-error (tn) "#\\x+000000063")
+
+(tn "R6RS (R5.92RS) string hex escapes")
+(assert-parse-error (tn) "\"\\x\"")
+(assert-parse-error (tn) "\"\\x6\"")
+(assert-parse-error (tn) "\"\\xf\"")
+(assert-parse-error (tn) "\"\\x63\"")
+(assert-parse-error (tn) "\"\\x063\"")
+(assert-parse-error (tn) "\"\\x0063\"")
+(assert-parse-error (tn) "\"\\x00063\"")
+(assert-parse-error (tn) "\"\\x0000063\"")
+(assert-parse-error (tn) "\"\\x00000063\"")
+(assert-parse-error (tn) "\"\\x000000063\"")
+
+(assert-parse-error (tn) "\"\\x;\"")
+(assert-parseable   (tn) "\"\\x6;\"")
+(assert-parseable   (tn) "\"\\xf;\"")
+(assert-parseable   (tn) "\"\\x63;\"")
+(assert-parseable   (tn) "\"\\x063;\"")
+(assert-parseable   (tn) "\"\\x0063;\"")
+(assert-parseable   (tn) "\"\\x00063;\"")
+(assert-parseable   (tn) "\"\\x0000063;\"")
+(assert-parseable   (tn) "\"\\x00000063;\"")
+(assert-parse-error (tn) "\"\\x000000063;\"")
+
+(assert-parse-error (tn) "\"\\x-\"")
+(assert-parse-error (tn) "\"\\x-6\"")
+(assert-parse-error (tn) "\"\\x-f\"")
+(assert-parse-error (tn) "\"\\x-63\"")
+(assert-parse-error (tn) "\"\\x-063\"")
+(assert-parse-error (tn) "\"\\x-0063\"")
+(assert-parse-error (tn) "\"\\x-00063\"")
+(assert-parse-error (tn) "\"\\x-0000063\"")
+(assert-parse-error (tn) "\"\\x-00000063\"")
+(assert-parse-error (tn) "\"\\x-000000063\"")
+
+(assert-parse-error (tn) "\"\\x-;\"")
+(assert-parse-error (tn) "\"\\x-6;\"")
+(assert-parse-error (tn) "\"\\x-f;\"")
+(assert-parse-error (tn) "\"\\x-63;\"")
+(assert-parse-error (tn) "\"\\x-063;\"")
+(assert-parse-error (tn) "\"\\x-0063;\"")
+(assert-parse-error (tn) "\"\\x-00063;\"")
+(assert-parse-error (tn) "\"\\x-0000063;\"")
+(assert-parse-error (tn) "\"\\x-00000063;\"")
+(assert-parse-error (tn) "\"\\x-000000063;\"")
+
+(assert-parse-error (tn) "\"\\x+\"")
+(assert-parse-error (tn) "\"\\x+6\"")
+(assert-parse-error (tn) "\"\\x+f\"")
+(assert-parse-error (tn) "\"\\x+63\"")
+(assert-parse-error (tn) "\"\\x+063\"")
+(assert-parse-error (tn) "\"\\x+0063\"")
+(assert-parse-error (tn) "\"\\x+00063\"")
+(assert-parse-error (tn) "\"\\x+0000063\"")
+(assert-parse-error (tn) "\"\\x+00000063\"")
+(assert-parse-error (tn) "\"\\x+000000063\"")
+
+(assert-parse-error (tn) "\"\\x+;\"")
+(assert-parse-error (tn) "\"\\x+6;\"")
+(assert-parse-error (tn) "\"\\x+f;\"")
+(assert-parse-error (tn) "\"\\x+63;\"")
+(assert-parse-error (tn) "\"\\x+063;\"")
+(assert-parse-error (tn) "\"\\x+0063;\"")
+(assert-parse-error (tn) "\"\\x+00063;\"")
+(assert-parse-error (tn) "\"\\x+0000063;\"")
+(assert-parse-error (tn) "\"\\x+00000063;\"")
+(assert-parse-error (tn) "\"\\x+000000063;\"")
+
 (total-report)
