@@ -2,6 +2,7 @@
 ;;  About    : unit test for SRFI-34
 ;;
 ;;  Copyright (C) 2005-2006 Kazuki Ohta <mover AT hct.zaq.ne.jp>
+;;  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
 ;;
 ;;  All rights reserved.
 ;;
@@ -30,18 +31,14 @@
 ;;  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-(load "./test/unittest.scm")
+(require-extension (unittest))
 
-(cond-expand
- (sigscheme
-  (use srfi-34))
- (else #t))
+(require-extension (srfi 8 34))
 
-(use srfi-8)
+(if (not (provided? "srfi-34"))
+    (test-skip "SRFI-34 is not enabled"))
 
-;; All tests in this file are passed against r2183 (new repository)
-
-;;(set! *test-track-progress* #t)
+(set! *test-track-progress* #f)
 
 (define tn test-name)
 
@@ -59,7 +56,12 @@
 ;;
 
 (tn "raise")
-
+;; no arg
+(assert-error (tn) (lambda ()
+                     (raise)))
+;; multiple values are not allowed
+(assert-error (tn) (lambda ()
+                     (raise (values 1 2 3))))
 ;; no guard or raw exception handler
 (assert-error (tn) (lambda ()
                      (raise 'exception)))
@@ -77,7 +79,7 @@
                           (lambda ()
                             (+ 1 (raise 'obj))))))
 
-;; handler a procedure but takes 2 arguments
+;; handler is a procedure but takes 2 arguments
 (my-assert-error (tn) (lambda ()
                         (with-exception-handler
                             eq?
@@ -91,7 +93,7 @@
                               'a-handler-must-not-return)
                           'an-error)))
 
-;; handler a procedure but takes an argument
+;; thunk is a procedure but takes an argument
 (my-assert-error (tn) (lambda ()
                         (with-exception-handler
                             (lambda (x)
@@ -103,7 +105,7 @@
 
 ;; Although the behavior when a handler returned is not specified in SRFI-34,
 ;; SigScheme should produce an error to prevent being misused by users.
-(if (provided? "sigscheme")
+(if sigscheme?
     (my-assert-error (tn) (lambda ()
                             (with-exception-handler
                                 (lambda (x)
@@ -135,14 +137,28 @@
                  (lambda ()
                    'success)))
 
+;; multiple values are allowed for thunk
+(assert-equal? (tn)
+               '(1 2 3)
+               (receive vals
+                   (with-exception-handler
+                       (lambda (x)
+                         'not-reaches-here)
+                     (lambda ()
+                       (values 1 2 3)))
+                 vals))
 
+
+;;
 ;; guard
+;;
+
 (tn "guard")
 (assert-equal? (tn)
                'exception
 	       (guard (condition
 		       (else
-			(assert-equal? "guard #2" 'an-error condition)
+			(assert-equal? (tn) 'an-error condition)
 			'exception))
                  (+ 1 (raise 'an-error))))
 
@@ -189,10 +205,10 @@
 
 ;; not matched against => and fall through to else
 (assert-equal? (tn)
-               #f
+               'else
                (guard (condition
                        ((assv condition '((a 1) (b 2))) => cadr)
-                       (else #f))
+                       (else 'else))
                  (raise 'c)))
 
 ;;
