@@ -4,6 +4,7 @@
 ;;  About    : unit test for SRFI-48
 ;;
 ;;  Copyright (C) 2006 YAMAMOTO Kengo <yamaken AT bp.iij4u.or.jp>
+;;  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
 ;;
 ;;  All rights reserved.
 ;;
@@ -34,11 +35,12 @@
 
 ;; All tests in this file are passed against r3170 (new repository)
 
-(load "./test/unittest.scm")
+(require-extension (unittest))
 
-(use srfi-6)
-(use srfi-38)
-(use srfi-48)
+(require-extension (srfi 6 38 48))
+
+(if (not (provided? "srfi-48"))
+    (test-skip "SRFI-48 is not enabled"))
 
 ;; test SRFI-28 compatible part of SRFI-48
 (load "./test/test-srfi28.scm")
@@ -63,6 +65,8 @@
 (assert-error  (tn) (lambda () (format "a"     "~s" "aBc")))
 (assert-error  (tn) (lambda () (format '(0 1)  "~s" '(0 1))))
 (assert-error  (tn) (lambda () (format '#(0 1) "~s" '#(0 1))))
+(assert-error  (tn) (lambda () (format "~")))
+(assert-error  (tn) (lambda () (format "a~")))
 
 (tn "SRFI-48 format explicit port")
 (assert-equal? (tn)
@@ -204,70 +208,69 @@
 
 (if (and (symbol-bound? 'greatest-fixnum)
          (symbol-bound? 'least-fixnum))
-    (let ((greatest (number->string (greatest-fixnum))))
-      (cond
-       ((string=? greatest "134217727")
-        (tn "format 28bit fixnum")
-        (assert-equal? (tn)
-                       "-134217728"
-                       (format "~d" (least-fixnum)))
-        (assert-equal? (tn)
-                       "111111111111111111111111111"
-                       (format "~b" (greatest-fixnum)))
-        (assert-equal? (tn)
-                       "-1000000000000000000000000000"
-                       (format "~b" (least-fixnum)))
-        (assert-equal? (tn)
-                       "-111111111111111111111111111"
-                       (format "~b" (+ (least-fixnum) 1))))
+    (case fixnum-bits
+      ((28)
+       (tn "format 28bit fixnum")
+       (assert-equal? (tn)
+                      "-134217728"
+                      (format "~d" (least-fixnum)))
+       (assert-equal? (tn)
+                      "111111111111111111111111111"
+                      (format "~b" (greatest-fixnum)))
+       (assert-equal? (tn)
+                      "-1000000000000000000000000000"
+                      (format "~b" (least-fixnum)))
+       (assert-equal? (tn)
+                      "-111111111111111111111111111"
+                      (format "~b" (+ (least-fixnum) 1))))
 
-       ((string=? greatest "2147483647")
-        (tn "format 32bit fixnum")
-        (assert-equal? (tn)
-                       "-2147483648"
-                       (format "~d" (least-fixnum)))
-        (assert-equal? (tn)
-                       "1111111111111111111111111111111"
-                       (format "~b" (greatest-fixnum)))
-        (assert-equal? (tn)
-                       "-10000000000000000000000000000000"
-                       (format "~b" (least-fixnum)))
-        (assert-equal? (tn)
-                       "-1111111111111111111111111111111"
-                       (format "~b" (+ (least-fixnum) 1))))
+      ((32)
+       (tn "format 32bit fixnum")
+       (assert-equal? (tn)
+                      "-2147483648"
+                      (format "~d" (least-fixnum)))
+       (assert-equal? (tn)
+                      "1111111111111111111111111111111"
+                      (format "~b" (greatest-fixnum)))
+       (assert-equal? (tn)
+                      "-10000000000000000000000000000000"
+                      (format "~b" (least-fixnum)))
+       (assert-equal? (tn)
+                      "-1111111111111111111111111111111"
+                      (format "~b" (+ (least-fixnum) 1))))
 
-       ((string=? greatest "576460752303423487")
-        (tn "format 60bit fixnum")
-        (assert-equal? (tn)
-                       "-576460752303423488"
-                       (format "~d" (least-fixnum)))
-        (assert-equal? (tn)
-                       "11111111111111111111111111111111111111111111111111111111111"
-                       (format "~b" (greatest-fixnum)))
-        (assert-equal? (tn)
-                       "-100000000000000000000000000000000000000000000000000000000000"
-                       (format "~b" (least-fixnum)))
-        (assert-equal? (tn)
-                       "-11111111111111111111111111111111111111111111111111111111111"
-                       (format "~b" (+ (least-fixnum) 1))))
+      ((60)
+       (tn "format 60bit fixnum")
+       (assert-equal? (tn)
+                      "-576460752303423488"
+                      (format "~d" (least-fixnum)))
+       (assert-equal? (tn)
+                      "11111111111111111111111111111111111111111111111111111111111"
+                      (format "~b" (greatest-fixnum)))
+       (assert-equal? (tn)
+                      "-100000000000000000000000000000000000000000000000000000000000"
+                      (format "~b" (least-fixnum)))
+       (assert-equal? (tn)
+                      "-11111111111111111111111111111111111111111111111111111111111"
+                      (format "~b" (+ (least-fixnum) 1))))
 
-       ((string=? greatest "9223372036854775807")
-        (tn "format 64bit fixnum")
-        (assert-equal? (tn)
-                       "-9223372036854775808"
-                       (format "~d" (least-fixnum)))
-        (assert-equal? (tn)
-                       "111111111111111111111111111111111111111111111111111111111111111"
-                       (format "~b" (greatest-fixnum)))
-        (assert-equal? (tn)
-                       "-1000000000000000000000000000000000000000000000000000000000000000"
-                       (format "~b" (least-fixnum)))
-        (assert-equal? (tn)
-                       "-111111111111111111111111111111111111111111111111111111111111111"
-                       (format "~b" (+ (least-fixnum) 1))))
+      ((64)
+       (tn "format 64bit fixnum")
+       (assert-equal? (tn)
+                      "-9223372036854775808"
+                      (format "~d" (least-fixnum)))
+       (assert-equal? (tn)
+                      "111111111111111111111111111111111111111111111111111111111111111"
+                      (format "~b" (greatest-fixnum)))
+       (assert-equal? (tn)
+                      "-1000000000000000000000000000000000000000000000000000000000000000"
+                      (format "~b" (least-fixnum)))
+       (assert-equal? (tn)
+                      "-111111111111111111111111111111111111111111111111111111111111111"
+                      (format "~b" (+ (least-fixnum) 1))))
 
-       (else
-        (error "unknown int bitwidth")))))
+      (else
+       (error "unknown int bitwidth"))))
 
 (tn "format ~c")
 (assert-error  (tn) (lambda () (format "~c")))
@@ -309,6 +312,9 @@
 (assert-error  (tn) (lambda () (format "~-0,-1f" 1)))
 (assert-error  (tn) (lambda () (format "~-1,-0f" 1)))
 (assert-error  (tn) (lambda () (format "~-1,-1f" 1)))
+(assert-error  (tn) (lambda () (format "~,f"     1)))
+(assert-error  (tn) (lambda () (format "~,1f"    1)))
+(assert-error  (tn) (lambda () (format "~1,f"    1)))
 (assert-equal? (tn) "-100" (format "~f" -100))
 (assert-equal? (tn) "-10"  (format "~f" -10))
 (assert-equal? (tn) "-1"   (format "~f" -1))
@@ -367,6 +373,16 @@
 (assert-equal? (tn) "   10" (format "~5f" 10))
 (assert-equal? (tn) "  100" (format "~5f" 100))
 
+(if (not testing-format+?)
+    (begin
+      (assert-equal? (tn) "-100" (format "~0,0f" -100))
+      (assert-equal? (tn) "-10"  (format "~0,0f" -10))
+      (assert-equal? (tn) "-1"   (format "~0,0f" -1))
+      (assert-equal? (tn) "0"    (format "~0,0f" 0))
+      (assert-equal? (tn) "1"    (format "~0,0f" 1))
+      (assert-equal? (tn) "10"   (format "~0,0f" 10))
+      (assert-equal? (tn) "100"  (format "~0,0f" 100))))
+
 (if (not testing-format+?)
     (begin
       (assert-equal? (tn) " -100" (format "~05f" -100))
@@ -644,6 +660,17 @@
 (assert-equal? (tn)
                "#(#t 123 #\\a \"aBc\" (0))"
                (format "~y" '#(#t 123 #\a "aBc" (0))))
+(tn "format ~y with explicit port to pretty-print")
+(let ((p (open-output-string)))
+  (format p "~y" 123)
+  (assert-equal? (tn) "123" (get-output-string p)))
+(define pretty-print write)
+(let ((p (open-output-string)))
+  (format p "~y" 123)
+  (assert-equal? (tn) "123" (get-output-string p)))
+(define pretty-print #f)
+(let ((p (open-output-string)))
+  (assert-error  (tn) (lambda () (format p "~y" 123))))
 
 (tn "format ~t")
 (assert-error  (tn) (lambda () (format "~t" #t)))
