@@ -2,6 +2,7 @@
 ;;  About    : unit test for R5RS hygienic macro
 ;;
 ;;  Copyright (C) 2006 Jun Inoue <jun.lambda@gmail.com>
+;;  Copyright (c) 2007 SigScheme Project <uim-en AT googlegroups.com>
 ;;
 ;;  All rights reserved.
 ;;
@@ -29,8 +30,10 @@
 ;;  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 ;;  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
-(load "./test/unittest.scm")
+(require-extension (unittest))
 
+(if (not (symbol-bound? 'syntax-rules))
+    (test-skip "R5RS hygienic macros is not enabled"))
 
 ;; Uncomment these if the interpreter crashes somewhere
 
